<?php

use envoqon\App;
use envoqon\db\QueryBuilder;
use envoqon\utils\Date;
use expertinvoice\application\models\Customer;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;
use expertinvoice\application\models\Template;
use expertinvoice\application\utils\Helper;


class ExpertInvoicePlugin {

    protected static $instance;

    
    public static $app;

    public function __construct() {
        self::$app = App::init(array(
                    'basePath' => plugin_dir_path(__FILE__) . 'application',
                    'pluginUrl' => plugin_dir_url(__FILE__),
                    'name' => 'expert-invoice',
                    'ajaxAction' => 'expert_invoice_ajax_action'
        ));
        add_action('admin_menu', array(get_class($this), 'loadMenus'));
        add_action('plugins_loaded', array(get_class($this), 'loadMessages'));
        add_action('admin_enqueue_scripts', array(get_class($this), 'loadAdminScripts'));
        add_action('init', array(get_class($this), 'loadFrontEndScripts'));
        add_action('admin_head', array(get_class($this), 'adminHeader'));
        add_action('admin_footer', array(get_class($this), 'adminFooter'));
        add_shortcode('expert-invoice', array(get_class($this), 'shortcode'));
        Template::model()->createPostType();
    }

    public static function loadMessages() {
        $plugin_rel_path = basename(dirname(__FILE__)) . '/application/languages/';
        load_plugin_textdomain('expert-invoice', false, $plugin_rel_path);
    }

    public static function adminHeader() {
        $screen = get_current_screen();
        if (strpos($screen->id, 'expert-invoice') !== false && strpos($screen->id, 'paypal') == false) {
            echo '<script type="text/javascript">'
            . 'var global = global || {};'
            . 'global.appBaseUrl = ' . json_encode(admin_url('admin-ajax.php')) . ';'
            . 'global.pluginBaseUrl = ' . json_encode(ExpertInvoicePlugin::$app->getPluginUrl()) . ';'
            . 'global.ajaxAction = ' . json_encode(ExpertInvoicePlugin::$app->getAjaxAction()) . ';'
            . 'global.dateFormat = ' . json_encode(Date::convertToMoment(Option::getAppOption()->dateFormat)) . ';'
            . '</script>';
        }
    }

    public static function adminFooter() {
        $screen = get_current_screen();
        if (strpos($screen->id, 'expert-invoice') !== false && strpos($screen->id, 'paypal') == false) {
            echo '<div class="spinner is-active ajax-processing" style="display:none"></div>';
        }
    }

    public static function loadMenus() {
        if (Option::getAppOption()->hasAcces()) {
            if(empty($GLOBALS['admin_page_hooks']['expert-invoice'])){
                add_menu_page('Expert Invoice', 'Expert Invoice', 'read', 'expert-invoice', array(self::class, 'run'), 'dashicons-awards', 25);             }
            add_submenu_page('expert-invoice', 'Dashboard', 'Dashboard', 'read', 'expert-invoice', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Products', 'Products', 'read', 'expert-invoice-product', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Invoices', 'Invoices', 'read', 'expert-invoice-invoice', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Quotes', 'Quotes', 'read', 'expert-invoice-quote', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Refunds', 'Refunds', 'read', 'expert-invoice-refund', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Payments', 'Payments', 'read', 'expert-invoice-payment', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Customers', 'Customers', 'read', 'expert-invoice-customer', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Customer Categories', 'Customer Categories', 'read', 'expert-invoice-customer-category', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Taxes', 'Taxes', 'read', 'expert-invoice-tax', array(self::class, 'run'));
            add_submenu_page('expert-invoice', 'Email Templates', 'Email Templates', 'read', 'edit.php?post_type=expert_template');
            add_submenu_page('expert-invoice', 'Settings', 'Settings', 'read', 'expert-invoice-option', array(self::class, 'run'));
            do_action('expert-invoice-submenu-page');
        }
    }

    public static function loadAdminScripts() {
        $wp_scripts = wp_scripts();
        wp_enqueue_style(
                'jquery-ui-theme-smoothness', sprintf(
                        '//ajax.googleapis.com/ajax/libs/jqueryui/%s/themes/smoothness/jquery-ui.css',                         $wp_scripts->registered['jquery-ui-core']->ver
                )
        );
        wp_register_script('knockout-3.4.2', plugins_url('js/vendors/knockout-3.4.2.js', __FILE__), array('jquery'), false, true);
        wp_register_script('knockout.mapping.js', plugins_url('js/vendors/knockout.mapping.js', __FILE__), array('jquery'), false, true);
        wp_register_script('knockout.validation.', plugins_url('js/vendors/knockout.validation.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('mynotify.min.js', plugins_url('js/vendors/notify.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('moment.min.js', plugins_url('js/vendors/moment.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('typeahead.jquery.min', plugins_url('js/vendors/typeahead.jquery.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('expert-invoice-ko.extensions.js', plugins_url('js/ko.extensions.min.js', __FILE__), array('jquery', 'knockout-3.4.2'), false, true);
        wp_register_script('expert-invoice-main.js', plugins_url('js/main.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('expert-invoice-viewmodel', plugins_url('js/viewmodel.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('expert-invoice-controller', plugins_url('js/controller.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('expert-invoice-dashboard-widget', plugins_url('js/dashboard-widget.min.js', __FILE__), array('jquery'), false, true);
        wp_register_script('expert-invoice-sale-min', plugins_url('js/sale.min.js', __FILE__), array('jquery'), false, true);
        wp_register_style('expert-invoice-style', plugins_url('css/style.min.css', __FILE__));
        wp_register_style('expert-invoice-print', plugins_url('css/print.min.css', __FILE__));
        wp_register_style('expert-invoice-receipt', plugins_url('css/invoice.css', __FILE__));
        wp_register_style('expert-invoice-notify.min.css', plugins_url('css/notify.min.css', __FILE__));
    }

    public static function loadFrontEndScripts() {
        wp_register_style('expert-invoice-receipt', plugins_url('css/invoice.css', __FILE__));
    }

    public static function run() {
        wp_enqueue_media();
        wp_enqueue_script('jquery-ui-autocomplete');
        wp_enqueue_script('jquery-ui-datepicker');
        wp_enqueue_script('jquery-ui-dialog');
        wp_enqueue_script('jquery-ui-tabs');
        wp_enqueue_script('jquery-effects-fold');
        wp_enqueue_script('knockout-3.4.2');
        wp_enqueue_script('knockout.mapping.js');
        wp_enqueue_script('knockout.validation.');
        wp_enqueue_script('mynotify.min.js');
        wp_enqueue_script('moment.min.js');
        wp_enqueue_script('typeahead.jquery.min');
        wp_enqueue_script('expert-invoice-ko.extensions.js');
        wp_enqueue_script('expert-invoice-main.js');
        wp_enqueue_script('expert-invoice-viewmodel');
        wp_enqueue_script('expert-invoice-controller');
        wp_enqueue_script('expert-invoice-dashboard-widget');
        wp_enqueue_script('expert-invoice-sale-min');
        wp_enqueue_style('expert-invoice-style');
        wp_enqueue_style('expert-invoice-print');
        wp_enqueue_style('expert-invoice-receipt');
        wp_enqueue_style('expert-invoice-notify.min.css');
        ExpertInvoicePlugin::$app->runAdmin();
    }

    
    public static function init() {
        if (static::$instance == null) {
            static::$instance = new ExpertInvoicePlugin();
        }
        return static::$instance;
    }

    public static function shortcode($atts = [], $content = null) {
                $id = isset($_GET['invoiceid']) ? $_GET['invoiceid'] : 0;
        wp_enqueue_style('expert-invoice-receipt');
        ob_start();
        $model = Invoice::model()->findByPk($id);
        if ($model != null) {
            self::render($model);
        }
        return $content . ob_get_clean();
    }

    private static function doPaymentMethods($id) {
        $paymentMethods = QueryBuilder::get()->select('shortcode')->from('{{expertinvoice_payment_method}}')->eq('online', 1)->queryColumn();
        foreach ($paymentMethods as $shortcode) {
            echo do_shortcode('[' . $shortcode . ' id="' . $id . '"]');
        }
    }

    public static function render($model) {
        $customer = Customer::model()->findByPk($model->customerId);
        $filename = $model->type == Invoice::TYPE_INVOICE ? 'invoice/_template.php' : 'quote/_template.php';
        $file = ExpertInvoicePlugin::$app->getViewPath() . DIRECTORY_SEPARATOR . $filename;
        $source = Helper::renderFile($file, array('model' => $model, 'customer' => $customer, 'items' => $model->getInvoiceItems()), true);
        libxml_use_internal_errors(true);
        $dom = new DOMDocument;
        $dom->loadHTML($source);
        libxml_use_internal_errors(false);
        $node = $dom->getElementById('receipt-template');
        ?>
        <div class="expert-invoice-paypal">
        <?php echo $node->ownerDocument->saveHTML($node); ?>
            <div class="payment-gateways">
        <?php self::doPaymentMethods($model->id); ?>
            </div>
        </div>

                <?php
            }

        }

        ExpertInvoicePlugin::init();


        