<?php

use envoqon\web\Controller;
use envoqon\web\Html;
use envoqon\web\HttpException;
use expertinvoice\application\models\Customer;
use expertinvoice\application\models\CustomerCategory;
use expertinvoice\application\models\Option;

class CustomerCategoryController extends Controller {

   public function filters() {
        return array(
            'accessControl'
        );
    }
    
    
    public function accessRules() {
        return array(
            array('allow', 
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny', 
                'users' => array('*'),
            ),
        );
    }

    
    public function actionView($id) {
        $model = $this->loadModel($id);
        echo json_encode(array('code' => 200, 'data' => $model->toArray()));
    }

    
    public function actionCreate() {
        $model = new CustomerCategory();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'CustomerCategory saved'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionUpdate($id) {
        $model = $this->loadModel($id);
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'CustomerCategory updated'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionDelete($id) {
        $model = $this->loadModel($id);
        if ((int) ($count = Customer::model()->count('categoryId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500,'message' => "Could not delete selected Category(s). It has been assigned to {$count} customers(s)."));
            return;
        }
        $model->delete();
        echo json_encode(array('id' => $id, 'message' => 'Selected record deleted'));
    }

    
    public function actionIndex() {
        $model = new CustomerCategory('search');
        $model->unsetAttributes();          if (isset($_GET['CustomerCategory'])) {
            $model->attributes = $_GET['CustomerCategory'];
        }
        $this->pageTitle = 'Customer Categories';
        $this->render('index', array(
            'model' => $model,
        ));
    }

    
    public function loadModel($id) {
        $model = CustomerCategory::model()->findByPk($id);
        if ($model === null) {
            throw new HttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

}
