<?php

use envoqon\web\Controller;
use envoqon\web\Html;
use envoqon\web\HttpException;
use expertinvoice\application\models\Customer;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;

class CustomerController extends Controller {

    public function filters() {
        return array(
            'accessControl'
        );
    }
    
    
    public function accessRules() {
        return array(
            array('allow', 
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny', 
                'users' => array('*'),
            ),
        );
    }
    
    public function actionSuggest(){
        if (isset($_GET['term']) && ($keyword = trim($_GET['term'])) !== '') {
            $data = Customer::model()->suggest($keyword);
            if ($data !== array()) {
                echo json_encode($data);
            }
        }
    }

    
    public function actionView($id) {
        $model = $this->loadModel($id);
        echo json_encode(array('code' => 200, 'data' => $model->toArray()));
    }

    
    public function actionCreate() {
        $model = new Customer();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'Customer saved'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionUpdate($id) {
        $model = $this->loadModel($id);
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'Customer updated'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionDelete($id) {
        if ((int) ($count = Invoice::model()->count('customerId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500,'message' => "Could not delete selected Customer. The customer has {$count} invoice(s)."));
            return;
        }
        $this->loadModel($id)->delete();
        echo json_encode(array('id' => $id, 'message' => 'Selected record deleted'));
    }

    
    public function actionIndex() {
        $model = new Customer('search');
        $model->unsetAttributes();          if (isset($_GET['Customer'])) {
            $model->attributes = $_GET['Customer'];
        }
        $this->pageTitle = 'Customer';
        $this->render('index', array(
            'model' => $model,
        ));
    }

    
    public function loadModel($id) {
        $model = Customer::model()->findByPk($id);
        if ($model === null) {
            throw new HttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

}
