<?php

use envoqon\db\QueryBuilder;
use envoqon\utils\Date;
use envoqon\web\Controller;
use envoqon\web\Html;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;
use expertinvoice\application\utils\Helper;


class ExpertInvoiceController extends Controller{
    
    public function filters() {
        return array(
            'accessControl'
        );
    }
    
    
    public function accessRules() {
        return array(
            array('allow', 
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny', 
                'users' => array('*'),
            ),
        );
    }
    
    public function actionIndex() {
        $this->pageTitle = 'Dashboard';
        $this->render('dashboard');
    }
    
     public function actionSalesDashboard($report, $period) {
        switch ($report) {
            case 'invoiceAndQuotes':
                $this->invoiceAndQuotes($period);
                break;
            case 'invoiceStatus':
                $this->invoiceStatus($period);
                break;
            case 'totalSales':
                $this->totalSales($period);
                break;
            case 'totalPaid':
                $this->totalPaid($period);
                break;
            case 'paymentDue':
                $this->paymentDue($period);
                break;
            case 'customers':
                $this->topTenCustomers($period);
                break;
            case 'itemsSold':
                $this->topTenItems($period);
                break;
            case 'refund':
                $this->totalRefund($period);
                break;
            case 'belowWarning':
                $this->totalItemsBelowWarning($period);
                break;
            default:
                break;
        }
    }
    
    public static function getStatusLabel($status) {
        switch ($status) {
            case Invoice::STATUS_UNPAID:
                return __('invoice.unpaid', 'expert-invoice');
            case Invoice::STATUS_PARTIALLY_PAID:
                return __('invoice.partialpaid', 'expert-invoice');
            case Invoice::STATUS_PAID:
                return __('invoice.paid', 'expert-invoice');
        }
    }
    
    public static function getTypeLabel($type) {
        switch ($type) {
            case Invoice::TYPE_INVOICE:
                return 'Invoice';
            default:
                return 'Quote';
        }
    }
    
    private function invoiceAndQuotes($period) {
        $range = $this->getDateRange($period);
        $rows = QueryBuilder::get()
                ->select('COUNT(id) as total, type')
                ->from('{{expertinvoice_invoice}}')
                ->between('invoicedDate', $range[0], $range[1])
                ->group('type')
                ->query();
        $data = array();
        $data[] = array('Invoice & Quotes', 'Total',array( 'role'=> 'style' ));
        foreach ($rows as $key => $row) {
            $data[] = array(
                self::getTypeLabel($row['type']), floatval($row['total']),Helper::getColor($key)
            );
        }
        $options = array(
            'title' => '',
        );
        echo json_encode(array('data' => $data, 'options' => $options));
    }
    
    private function invoiceStatus($period) {
        $range = $this->getDateRange($period);
        $rows = QueryBuilder::get()
                ->select('COUNT(id) as total, status')
                ->from('{{expertinvoice_invoice}}')
                ->between('invoicedDate', $range[0], $range[1])
                ->eq('type', Invoice::TYPE_INVOICE)
                ->group('status')
                ->query();
        $data = array();
        $data[] = array('Status', 'Total',array('role'=>'style'));
        foreach ($rows as $key => $row) {          
            $data[] = array(
                self::getStatusLabel($row['status']), floatval($row['total']), Helper::getColor($key)
            );
        }
        $options = array(
            'title' => '',
            'pieHole' => 0.4,
            'vAxis' => [
                'title' => 'Total',
                'minValue' => 0,
                'gridlines' => array(
                    'color' => '#e1e1e3',
                )
            ],
            'hAxis' => [
                'title' => 'Status',
            ]
        );
        echo json_encode(array('data' => $data, 'options' => $options));
    }

    private function topTenCustomers($period) {
        $range = $this->getDateRange($period);
        $sort = isset($_GET['sort']) ? $_GET['sort'] : 'value';
        $query = QueryBuilder::get()
                ->select("c.name,c.email,c.email,count(si.quantity) as unit,s.amountDue")
                ->from('{{expertinvoice_customer}}', 'c')
                ->join('{{expertinvoice_invoice}}  s', 'c.id=s.customerId')
                ->leftJoin('{{expertinvoice_invoice_item}} si', 's.id=si.invoiceId')
                ->group('c.id')
                ->between('s.invoicedDate', $range[0], $range[1])
                ->limit(10);
        if ($sort == 'value') {
            $query->order('s.amountDue DESC');
        }
        else {
            $query->order('unit DESC');
        }
        $rows = $query->queryAll();
        $empty = array();
        echo '<table><thead><tr>';
        echo Html::tag('th', $empty, 'Customer');
        echo Html::tag('th', $empty, 'Unit Bought');
        echo Html::tag('th', $empty, 'Value');
        echo '</tr></thead><tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            $name = '<div>' . $row['name']. '</div><div>Email: ' . $row['email'] . '</div>';
            echo Html::tag('td', $empty, $name);
            echo Html::tag('td', $empty, $row['unit']);
            echo Html::tag('td', $empty, $row['amountDue']);
            echo '</tr>';
        }
        echo '</tbody></table>';
    }

    private function topTenItems($period) {
        $range = $this->getDateRange($period);
        $sort = isset($_GET['sort']) ? $_GET['sort'] : 'value';
        $query = QueryBuilder::get()
                ->select("i.name,i.sku,count(si.quantity) as unit,s.amountDue")
                ->from('{{expertinvoice_product}}', 'i')
                ->join('{{expertinvoice_invoice_item}} si', 'i.id=si.productId')
                ->join('{{expertinvoice_invoice}}  s', 's.id=si.invoiceId')
                ->group('i.id')
                ->between('s.invoicedDate', $range[0], $range[1])
                ->limit(10);
        if ($sort == 'value') {
            $query->order('s.amountDue DESC');
        }
        else {
            $query->order('unit DESC');
        }
        $rows = $query->queryAll();
        $empty = array();
        echo '<table><thead><tr>';
        echo Html::tag('th', $empty, 'Item');
        echo Html::tag('th', $empty, 'Unit Sold');
        echo Html::tag('th', $empty, 'Value');
        echo '</tr></thead><tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            $name = '<div>' . $row['name'] . '</div><p class="sub_text">' . $row['sku'] . '</p>';
            echo Html::tag('td', $empty, $name);
            echo Html::tag('td', $empty, $row['unit']);
            echo Html::tag('td', $empty, $row['amountDue']);
            echo '</tr>';
        }
        echo '</tbody></table>';
    }

    private function totalSales($period) {
        $range = $this->getDateRange($period);
        $total = QueryBuilder::get()
                ->select('COUNT(id)')
                ->from('{{expertinvoice_invoice}}')
                ->between('invoicedDate', $range[0], $range[1])
                ->eq('type', Invoice::TYPE_INVOICE)
                ->queryScalar();
        $totalAmount = QueryBuilder::get()
                ->select('SUM(amountDue)')
                ->from('{{expertinvoice_invoice}}')
                ->eq('type', Invoice::TYPE_INVOICE)
                ->between('invoicedDate', $range[0], $range[1])
                ->queryScalar();
        echo '<p>' . Option::getAppOption()->formatMoney($totalAmount) . '</p>'
        . '<p class="sub_text">' . Option::getAppOption()->formatInteger($total) . ' sales</p>';
    }
    
    private function totalPaid($period) {
        $range = $this->getDateRange($period);
        $total = QueryBuilder::get()
                ->select('COUNT(id)')
                ->from('{{expertinvoice_payment}}')
                ->between('datePaid', $range[0], $range[1])
                ->queryScalar();
        $totalAmount = QueryBuilder::get()
                ->select('SUM(amount)')
                ->from('{{expertinvoice_payment}}')
                ->between('datePaid', $range[0], $range[1])
                ->queryScalar();
        echo '<p>' . Option::getAppOption()->formatMoney($totalAmount) . '</p>'
        . '<p class="sub_text">' . Option::getAppOption()->formatInteger($total) . ' payments</p>';
    }

    private function paymentDue($period) {
        $range = $this->getDateRange($period);
        $totalRefund = QueryBuilder::get()
                ->select('SUM(amount)')
                ->from('{{expertinvoice_refund}}')
                ->between('refundDate', $range[0], $range[1])
                ->queryScalar();
        $totalPayment = QueryBuilder::get()
                ->select('SUM(amount)')
                ->from('{{expertinvoice_payment}}')
                ->between('datePaid', $range[0], $range[1])
                ->queryScalar();
        $totalAmount = QueryBuilder::get()
                ->select('SUM(amountDue)')
                ->from('{{expertinvoice_invoice}}')
                ->eq('type', Invoice::TYPE_INVOICE)
                ->between('invoicedDate', $range[0], $range[1])
                ->queryScalar();
        echo '<p>' . Option::getAppOption()->formatMoney($totalAmount + $totalRefund - $totalPayment) . '</p>';
    }

    private function totalRefund($period) {
        $range = $this->getDateRange($period);
        $totalItems = QueryBuilder::get()
                ->select('COUNT(id)')
                ->from('{{expertinvoice_refund}}')
                ->between('refundDate', $range[0], $range[1])
                ->queryScalar();
        $totalAmount = QueryBuilder::get()
                ->select('SUM(amount)')
                ->from('{{expertinvoice_refund}}')
                ->between('refundDate', $range[0], $range[1])
                ->queryScalar();
        echo '<p>' . Option::getAppOption()->formatMoney($totalAmount) . '</p>'
        . '<p class="sub_text">' . Option::getAppOption()->formatInteger($totalItems) . ' refunds</p>';
    }
    
     private function getDateRange($period) {
        switch ($period) {
            case 1:
                $today = Date::format(time(), Date::PATTERN_ISO_8601_DATE);
                return array($today, $today . ' 23:59:59');
            case 2:
                $dt = time();
                return array(
                    date('N', $dt) == 1 ? date('Y-m-d', $dt) : date('Y-m-d', strtotime('last monday', $dt)),
                    date('N', $dt) == 7 ? date('Y-m-d', $dt) . ' 23:59:59' : date('Y-m-d', strtotime('next sunday', $dt)) . ' 23:59:59'
                );
            case 3:
                $dt = time();
                return array(
                    date('Y-m-d', strtotime('first day of this month', $dt)),
                    date('Y-m-d', strtotime('last day of this month', $dt)) . ' 23:59:59'
                );
            default:
                break;
        }
    }
}
