<?php

use envoqon\utils\Date;
use envoqon\web\Controller;
use envoqon\web\Html;
use envoqon\web\HttpException;
use expertinvoice\application\models\Customer;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;
use expertinvoice\application\models\Payment;
use expertinvoice\application\models\Refund;
use expertinvoice\application\models\Template;

class InvoiceController extends Controller {

    public function filters() {
        return array(
            'accessControl'
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules() {
        return array(
            array('allow',
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny',
                'users' => array('*'),
            ),
        );
    }

    public function actionSuggest() {
        if (isset($_GET['term']) && ($keyword = trim($_GET['term'])) !== '') {
            $data = Invoice::model()->suggest($keyword);
            if ($data !== array()) {
                echo json_encode($data);
            }
        }
    }

    /**
     * Displays a particular model.
     * @param integer $id the ID of the model to be displayed
     */
    public function actionView($id) {
        $model = $this->loadModel($id);
        $customer = Customer::model()->findByPk($model->customerId);
        $source = $this->renderPartial('_template', array('model' => $model, 'customer' => $customer, 'items' => $model->getInvoiceItems()), true);
        libxml_use_internal_errors(true);
        $dom = new DOMDocument;
        $dom->loadHTML($source);
        libxml_use_internal_errors(false);
        $node = $dom->getElementById('receipt-template');
        echo $node->ownerDocument->saveHTML($node);
    }

    public function actionCreate() {
        $this->pageTitle = 'Add Invoice';
        $this->render('pos');
    }

    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     */
    public function actionSave() {
        $model = new Invoice();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            $model->type = Invoice::TYPE_INVOICE;
            if ($model->validate()) {
                $transaction = $model->dbConnection->beginTransaction();
                try {
                    $model->save();
                    $model->saveItems(isset($_POST['items']) ? $_POST['items'] : []);
                    $transaction->commit();
                }
                catch (Exception $e) {
                    $transaction->rollback();
                    echo json_encode(array('code' => 500, 'message' => $e->getMessage()));
                    ExpertInvoicePlugin::$app->end();
                }
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => $this->createAjaxUrl('invoice/print', array('id' => $model->id))));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    public function actionPrint($id) {
        $model = $this->loadModel($id);
        echo $model->getInvoiceTemplate();
    }

    

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id) {
        $model = $this->loadModel($id);
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            $model->type = Invoice::TYPE_INVOICE; //FromQuote
            if ($model->save()) {
                $model->saveItems(isset($_POST['items']) ? $_POST['items'] : []);
                $model->deleteItems(isset($_POST['deletedItems']) ? $_POST['deletedItems'] : []);
                echo json_encode(array('code' => 200, 'id' => $model->id,
                    'message' => $this->createAjaxUrl('invoice/print', array('id' => $model->id))));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
            ExpertInvoicePlugin::$app->end();
        }
        else {
            $this->pageTitle = 'Update Invoice';
            $this->render('pos', array('items' => $model->getInvoiceItems(), 'model' => $model));
        }
    }

    /**
     * Duplicates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionDuplicate($id) {
        $model = $this->loadModel($id);
        $items = $model->getInvoiceItems();
        foreach($items as $item){
            $item->id = '';
        }
        $model->id = '';
        $model->receiptNumber = $model->getNewInvoiceNumber();
        $model->invoicedDate = Date::format(time(), Date::PATTERN_ISO_8601_DATE);
        $model->dueDate = Date::format(time(), Date::PATTERN_ISO_8601_DATE);
        $this->pageTitle = 'Add Invoice';
        $this->render('pos', array('items' => $items, 'model' => $model));
    }

    public function actionFromQuote($id) {
        $model = $this->loadModel($id);
        $model->type = Invoice::TYPE_INVOICE;
        $model->receiptNumber = $model->getNewInvoiceNumber();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                $model->saveItems(isset($_POST['items']) ? $_POST['items'] : []);
                $model->deleteItems(isset($_POST['deletedItems']) ? $_POST['deletedItems'] : []);
                echo json_encode(array('code' => 200, 'id' => $model->id,
                    'message' => $this->getInvoice($model, 'Invoice')));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
            ExpertInvoicePlugin::$app->end();
        }
        else {
            $this->pageTitle = 'Invoice';
            $this->render('pos', array('items' => $model->getInvoiceItems(), 'model' => $model));
        }
    }

    public function actionEmail($id) {
        $model = $this->loadModel($id);
        $customer = Customer::model()->findByPk($model->customerId);
        $title = ($model->type == Invoice::TYPE_INVOICE ? 'Invoice' : 'Quote');
        if ($customer && $customer->email) {
            $slug = $model->type == Invoice::TYPE_INVOICE ? 'expert-invoice-template' : 'expert-invoice-quote-template';
            $template = Template::model()->findByAttributes(array('post_name' => $slug));
            if ($template) {
                $model->sendEmail($customer, $template);
                echo json_encode(array('code' => 200, 'message' => $title . ' emailed'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => 'Email Template not found'));
            }
        }
    }

    

    public function actionNextReceiptNumber() {
        echo json_encode(array('code' => 200, 'message' => Invoice::model()->getNewInvoiceNumber()));
    }

    

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id) {
        $model = $this->loadModel($id);
        if ((int) ($count = Payment::model()->count('invoiceId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500, 'message' => "Could not delete selected Invoice(s). It has some payments."));
            return;
        }
        if ((int) ($count = Refund::model()->count('invoiceId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500, 'message' => "Could not delete selected Invoice(s). It has some payments."));
            return;
        }
        $model->delete();
        echo json_encode(array('id' => $id, 'message' => 'Selected record deleted'));
    }

    /**
     * Manages all models.
     */
    public function actionIndex() {
        $model = new Invoice('search');
        $model->unsetAttributes();  // clear any default values
        if (isset($_GET['Invoice'])) {
            $model->attributes = $_GET['Invoice'];
        }
        $this->pageTitle = 'Invoices';
        $this->render('index', array(
            'model' => $model,
        ));
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer $id the ID of the model to be loaded
     * @return Invoice the loaded model
     * @throws HttpException
     */
    public function loadModel($id) {
        $model = Invoice::model()->findByPk($id);
        if ($model === null) {
            throw new HttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

}
