<?php

use envoqon\web\Controller;
use envoqon\web\Html;
use envoqon\web\HttpException;
use expertinvoice\application\models\IDGenerator;
use expertinvoice\application\models\InvoiceItem;
use expertinvoice\application\models\Option;
use expertinvoice\application\models\Product;

class ProductController extends Controller {

    public function filters() {
        return array(
            'accessControl'
        );
    }
    
    
    public function accessRules() {
        return array(
            array('allow', 
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny', 
                'users' => array('*'),
            ),
        );
    }
    
    public function actionSuggest() {
        if (isset($_GET['term']) && ($keyword = trim($_GET['term'])) !== '') {
            $data = Product::model()->suggest($keyword);
            if ($data !== array()) {
                echo json_encode($data);
            }
        }
    }

    public function actionGenerateId() {
        $auto_id = 0;
        do {
            $auto_id = IDGenerator::generate();
        } while (Product::model()->exists('LOWER(sku)=LOWER(:sku)', array(':sku' => $auto_id)));
        echo json_encode(array('code' => 200, 'message' => 'ID generated successfully', 'genID' => $auto_id));
    }
    
    public function actionView($id) {
        $model = $this->loadModel($id);
        echo json_encode(array('code' => 200, 'data' => $model->toArray()));
    }

    
    public function actionCreate() {
        $model = new Product();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'Product saved'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionUpdate($id) {
        $model = $this->loadModel($id);
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'Product updated'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionDelete($id) {
        if ((int) ($count = InvoiceItem::model()->count('productId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500,'message' => "Could not delete selected Product. It is part of items sold."));
            return;
        }
        $this->loadModel($id)->delete();
        echo json_encode(array('id' => $id, 'message' => 'Selected record deleted'));
    }

    
    public function actionIndex() {
        $model = new Product('search');
        $model->unsetAttributes();          if (isset($_GET['Product'])) {
            $model->attributes = $_GET['Product'];
        }
        $this->pageTitle = 'Product';
        $this->render('index', array(
            'model' => $model,
        ));
    }

    
    public function loadModel($id) {
        $model = Product::model()->findByPk($id);
        if ($model === null) {
            throw new HttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

}
