<?php

use envoqon\utils\Date;
use envoqon\web\Html;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;
require_once(__DIR__.DIRECTORY_SEPARATOR. 'InvoiceController.php');

class QuoteController extends InvoiceController {


    public function filters() {
        return array(
            'accessControl'
        );
    }
    
    
    public function accessRules() {
        return array(
            array('allow', 
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny', 
                'users' => array('*'),
            ),
        );
    }
    
    public function actionCreate() {
        $this->pageTitle = 'Add Quote';
        $this->render('pos');
    }

    
    public function actionSave() {
        $model = new Invoice();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            $model->type = Invoice::TYPE_QUOTE;
            if ($model->validate()) {
                $transaction = $model->dbConnection->beginTransaction();
                try {
                    $model->save();
                    $model->saveItems(isset($_POST['items']) ? $_POST['items'] : []);
                    $transaction->commit();
                }
                catch (Exception $e) {
                    $transaction->rollback();
                    echo json_encode(array('code' => 500, 'message' => $e->getMessage()));
                    ExpertInvoicePlugin::$app->end();
                }
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => $this->createAjaxUrl('quote/print',array('id'=>$model->id))));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionUpdate($id) {
        $model = $this->loadModel($id);
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                $model->saveItems(isset($_POST['items']) ? $_POST['items'] : []);
                $model->deleteItems(isset($_POST['deletedItems']) ? $_POST['deletedItems'] : []);
                echo json_encode(array('code' => 200, 'id' => $model->id,
                    'message' => $this->createAjaxUrl('quote/print',array('id'=>$model->id))));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
        else {
            $this->pageTitle = 'Update Quote';
            $this->render('pos', array('items' => $model->getInvoiceItems(),'model'=>$model));
        }
    }
    
    
    public function actionDuplicate($id) {
        $model = $this->loadModel($id);
        $items = $model->getInvoiceItems();
        $model->id = '';
        $model->receiptNumber = 0;
        $model->invoicedDate = Date::format(time(), Date::PATTERN_ISO_8601_DATE);
        $model->dueDate = Date::format(time(), Date::PATTERN_ISO_8601_DATE);
        $this->pageTitle = 'Add Quote';
        $this->render('pos', array('items' => $items, 'model' => $model));
    }

    
    public function actionIndex() {
        $model = new Invoice('search');
        $model->unsetAttributes();          if (isset($_GET['Invoice'])) {
            $model->attributes = $_GET['Invoice'];
        }
        $this->pageTitle = 'Quotes';
        $this->render('index', array(
            'model' => $model,
        ));
    }

}
