<?php

use envoqon\web\Controller;
use envoqon\web\Html;
use envoqon\web\HttpException;
use expertinvoice\application\models\InvoiceItem;
use expertinvoice\application\models\Option;
use expertinvoice\application\models\Product;
use expertinvoice\application\models\Tax;

class TaxController extends Controller {

    public function filters() {
        return array(
            'accessControl'
        );
    }
    
    
    public function accessRules() {
        return array(
            array('allow', 
                'roles' => Option::getAppOption()->roles,
            ),
            array('deny', 
                'users' => array('*'),
            ),
        );
    }

    
    public function actionView($id) {
        $model = $this->loadModel($id);
        echo json_encode(array('code' => 200, 'data' => $model->toArray()));
    }

    
    public function actionCreate() {
        $model = new Tax();
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'Tax saved'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionUpdate($id) {
        $model = $this->loadModel($id);
        if (!empty($_POST)) {
            $model->setAttributes($_POST);
            if ($model->save()) {
                echo json_encode(array('code' => 200, 'id' => $model->id, 'message' => 'Tax updated'));
            }
            else {
                echo json_encode(array('code' => 500, 'message' => Html::errorSummary($model)));
            }
        }
    }

    
    public function actionDelete($id) {
        if ((int) ($count = Product::model()->count('taxId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500,'message' => "Could not delete selected Tax. It has been assigned to {$count} product(s)."));
            return;
        }
        if ((int) ($count = InvoiceItem::model()->count('taxId =:id', array(':id' => $id))) > 0) {
            echo json_encode(array('code' => 500,'message' => "Could not delete selected Tax. It has been assigned to {$count} sale(s)."));
            return;
        }
        $this->loadModel($id)->delete();
        echo json_encode(array('id' => $id, 'message' => 'Selected record deleted'));
    }

    
    public function actionIndex() {
        $model = new Tax('search');
        $model->unsetAttributes();          if (isset($_GET['Tax'])) {
            $model->attributes = $_GET['Tax'];
        }
        $this->pageTitle = 'Tax';
        $this->render('index', array(
            'model' => $model,
        ));
    }

    
    public function loadModel($id) {
        $model = Tax::model()->findByPk($id);
        if ($model === null) {
            throw new HttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

}
