<?php

namespace expertinvoice\application\models;

use envoqon\utils\Date;
use envoqon\db\ActiveRecord;
use envoqon\db\DbCriteria;
use envoqon\web\ActiveDataProvider;
use expertinvoice\application\models\Invoice;


class Customer extends ActiveRecord {

    private static $names;
    
    public function tableName() {
        return '{{expertinvoice_customer}}';
    }

    
    public function rules() {
                        return array(
            array('name, categoryId', 'required'),
            array('isActive, isTaxExempt, categoryId', 'numerical', 'integerOnly' => true),
            array('name, contactPerson', 'length', 'max' => 255),
            array('email, telephone1, telephone2', 'length', 'max' => 128),
            array('shippingAddress, notes', 'safe'),
                                    array('id, name, contactPerson, email, telephone1, telephone2, isActive, shippingAddress, notes, isTaxExempt, categoryId', 'safe', 'on' => 'search'),
        );
    }

    
    public function getCustomerCategory() {
        return CustomerCategory::model()->findByPk($this->getPrimaryKey());
    }

    
    public function getInvoices() {
        return Invoice::model()->findAllByAttributes(array('customerId' => $this->getPrimaryKey()));
    }

    public function getAttributeLabel($attribute) {
        return __('customer.' . $attribute, 'expert-invoice');
    }

    
    public function search() {
        $q = isset($_GET['q']) ? $_GET['q'] : null;
        $criteria = new DbCriteria;

        $criteria->compare('id', $q, false, 'OR');
        $criteria->compare('name', $q, true, 'OR');
        $criteria->compare('contactPerson', $q, true, 'OR');
        $criteria->compare('email', $q, true, 'OR');
        $criteria->compare('telephone1', $q, true, 'OR');
        $criteria->compare('telephone2', $q, true, 'OR');
        $criteria->compare('isActive', $q, false, 'OR');
        $criteria->compare('shippingAddress', $q, true, 'OR');
        $criteria->compare('notes', $q, true, 'OR');
        $criteria->compare('isTaxExempt', $q, false, 'OR');
        $criteria->compare('categoryId', $q, false, 'OR');

        return new ActiveDataProvider($this, array(
            'criteria' => $criteria,
        ));
    }

    
    public static function model($className = __CLASS__) {
        return parent::model($className);
    }
    
    protected function beforeSave() {
        if (parent::beforeSave()) {
            if ($this->isNewRecord) {
                $this->createdDate = Date::format(time(), Date::PATTERN_ISO_8601_DATETIME);
                $this->updatedDate = $this->createdDate;
            }
            else {
                $this->updatedDate = Date::format(time(), Date::PATTERN_ISO_8601_DATETIME);
            }
            return true;
        }
        return false;
    }
    
    public function suggest($keyword, $limit = 10) {
        $keyword = '%' . strtr($keyword, array('%' => '\%', '_' => '\_', '\\' => '\\\\')) . '%';
        $models = $this->findAll(array(
            'condition' => 't.name LIKE :keyword AND isActive=1',
            'select' => 't.id,t.name,t.shippingAddress,t.isTaxExempt,telephone1,email',
            'order' => 't.name',
            'limit' => $limit,
            'params' => array(
                ':keyword' => $keyword
            ),
        ));
        $names = array();
        foreach ($models as $tag) {
            $names[] = array('label' => $tag->name, 'value' => $tag->name, 'id' => $tag->id,'email'=>$tag->email,
                'shippingAddress'=>$tag->shippingAddress,'isTaxExempt'=>$tag->isTaxExempt,'telephone'=>$tag->telephone1);
        }
        return $names;
    }
    
    public function fields() {
        return array(
            'id',
            'name',
            'shippingAddress',
            'contactPerson',
            'email',
            'telephone1',
            'telephone2',
            'notes',
            'isTaxExempt',
            'categoryId',
            'isActive'
        );
    }
    
    public function listAll() {
        if (self::$names != null) {
            return self::$names;
        }
        $cat = $this->findAll(array('order' => 'name'));
        self::$names = array();
        foreach ($cat as $t) {
            self::$names[$t->id] = $t->name;
        }
        return self::$names;
    }

}
