<?php

namespace expertinvoice\application\models;

use envoqon\db\ActiveRecord;
use envoqon\db\DbCriteria;
use envoqon\db\QueryBuilder;
use envoqon\web\ActiveDataProvider;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Product;


class InvoiceItem extends ActiveRecord {
    
    public $isInvoiceItem = false;

    
    public function tableName() {
        return '{{expertinvoice_invoice_item}}';
    }

    
    public function rules() {
                        return array(
            array('invoiceId, productId,salePrice,taxId', 'required'),
            array('invoiceId, quantity, productId', 'numerical', 'integerOnly' => true),
            array('salePrice,discount,tax,taxRate', 'length', 'max' => 14),
                                    array('id, invoiceId, quantity, discount,tax,taxRate,salePrice, productId', 'safe', 'on' => 'search'),
        );
    }

    
    public function getProduct() {
        return Product::model()->findByPk($this->productId);
    }

    
    public function getInvoice() {
        return Invoice::model()->findByPk($this->invoiceId);
    }

    public function getAttributeLabel($attribute) {
        return __('invoiceItem.' . $attribute, 'expert-invoice');
    }

    
    public function search() {
        $q = isset($_GET['q']) ? $_GET['q'] : null;
        $criteria = new DbCriteria;

        $criteria->compare('id', $q, false, 'OR');
        $criteria->compare('invoiceId', $q, false, 'OR');
        $criteria->compare('quantity', $q, false, 'OR');
        $criteria->compare('salePrice', $q, true, 'OR');
        $criteria->compare('productId', $q, false, 'OR');

        return new ActiveDataProvider($this, array(
            'criteria' => $criteria,
        ));
    }

    
    public static function model($className = __CLASS__) {
        return parent::model($className);
    }

    public function fields() {
        return array(
            'productId',
            'name',
            'quantity',
            'salePrice',
            'tax',
            'taxRate',
            'discount',
            'id'
        );
    }
    
    public function getName() {
        return QueryBuilder::get()->select('name')->from('{{expertinvoice_product}}')->eq('id', $this->productId)->queryScalar();
    }
    
    protected function afterSave() {
        parent::afterSave();
        $product = Product::model()->findByPk($this->productId);
        if ($product && $this->isInvoiceItem) {
            $product->adjustStock(-(int)$this->quantity);
        }
    }
    
    protected function afterDelete() {
        parent::afterDelete();
        $product = Product::model()->findByPk($this->productId);
        if ($product && $this->isInvoiceItem) {
            $product->adjustStock((int)$this->quantity);
        }
    }

}
