<?php

namespace expertinvoice\application\models;

use envoqon\web\WPOption;


class Option extends WPOption {

    
    private static $_pref = null;
    public $invoiceStartNumber = 1;
    public $successPage = '';
    public $paymentPage = '';
    public $name = "";
    public $email = "";
    public $contactDetails = "";
    public $logo = "";
    public $currency = "$";
    public $receiptTitle = 'Receipt';
    public $quantityColumn = 'Qty';
    public $descriptionColumn = 'Description';
    public $issueDate = 'Issue Date';
    public $issueTime = 'Issue Time';
    public $decimalSymbol = ".";
    public $dateFormat = 'd M, Y';
    public $thousandsSymbol = ",";
    public $registeredNumber = '';
    public $taxIncluded = 1;
    public $footerComment = 'Thank you for your business';
    public $footerCommentAlignment = 'align-center';
    public $footerCommentFontSize = 'normal';
    public $unitPriceColumn = 'Unit Price';
    public $discountColumn = 'Discount';
    public $lineTotalColumn = 'Total';
    public $transactionNumber = 'TransactionRef';
    public $refundNumber = 'Refund Number';
    public $salesPerson = 'Salesperson';
    public $subTotal = 'Subtotal';
    public $totalAmount = 'Total Amount';
    public $amountTendered = 'Amount Paid';
    public $refundTitle = 'Receipt';
    public $change = 'Change Due';
    public $invoiceDueDate = 'Pay By';
    public $quoteDueDate = 'Valid Until';
    public $transactionReference = 'TransactionRef';
    public $roles = array('administrator');
    public static $fontSizes = array(
        'normal' => 'Normal',
        'small' => 'Small',
    );
    public static $alignments = array(
        'align-left' => 'Left',
        'align-center' => 'Center',
        'align-right' => 'Right',
    );
    public static $receiptText = array(
        'receiptTitle' => 'Receipt Title',
        'refundTitle' => 'Refund Title',
        'quantityColumn' => 'Quantity Column',
        'descriptionColumn' => 'Description Column',
        'unitPriceColumn' => 'Unit Price Column',
        'discountColumn' => 'Discount Column',
        'lineTotalColumn' => 'Line TotalColumn',
        'transactionNumber' => 'Transaction Number',
        'issueDate' => 'Issue Date',
        'issueTime' => 'Issue Time',
        'salesPerson' => 'Sales Person',
        'subTotal' => 'Subtotal',
        'totalAmount' => 'Total Amount',
        'amountTendered' => 'Amount Tendered',
        'change' => 'Change',
        'transactionReference' => 'Transaction Reference',
        'refundNumber' => 'Refund Number',
        'invoiceDueDate' => 'Invoice Due Date',
        'quoteDueDate' => ' Quote Due Date'
    );

    public function __construct($app) {
        parent::__construct($app);
    }

    
    public function rules() {
                        return array(
            array('name,currency,thousandsSymbol,dateFormat,decimalSymbol,roles,invoiceStartNumber', 'required'),
            array('email', 'email'),
            array('taxIncluded,successPage,paymentPage', 'numerical', 'integerOnly' => true),
            array('name,quoteDueDate,invoiceDueDate, registeredNumber, logo, receiptTitle, refundTitle,quantityColumn, descriptionColumn, unitPriceColumn, discountColumn, lineTotalColumn, transactionNumber, issueDate, issueTime, salesPerson, subTotal, totalAmount, amountTendered, change, transactionReference, refundNumber', 'length', 'max' => 255),
            array('currency', 'length', 'max' => 8),
            array('footerCommentAlignment, footerCommentFontSize', 'length', 'max' => 16),
            array('contactDetails, footerComment,logo', 'safe'),
                                    array('id, name, registeredNumber, contactDetails, logo, taxIncluded, currency, footerComment, footerCommentAlignment, footerCommentFontSize,receiptTitle, quantityColumn, descriptionColumn, unitPriceColumn, discountColumn, lineTotalColumn, transactionNumber, issueDate, issueTime, salesPerson, subTotal, totalAmount, amountTendered, change, transactionReference, refundNumber', 'safe', 'on' => 'search'),
        );
    }

    public function hasLogo() {
        return !empty(Option::getAppOption()->logo);
    }

    public function getLogoLink() {
        return $this->logo;
    }

    public function formatNumber($number) {
        return number_format($number, 2, $this->decimalSymbol, $this->thousandsSymbol);
    }

    public function formatInteger($number) {
        return number_format($number, 0, $this->decimalSymbol, $this->thousandsSymbol);
    }

    public function formatMoney($number) {
        return $this->currency . $this->formatNumber($number);
    }

    public function toArray() {
        $values = array();
        $attributes = $this->attributeNames();
        foreach ($attributes as $name) {
            $values[$name] = $this->$name;
        }
        return $values;
    }

    
    public static function getAppOption() {
        if (self::$_pref == null) {
            self::$_pref = new Option(\ExpertInvoicePlugin::$app);
            self::$_pref->init();
        }
        return self::$_pref;
    }

    public function getAttributeLabel($attribute) {
        return __('option.' . $attribute, 'expert-invoice');
    }

    public function getSuccessPageUrl() {
        return get_permalink($this->successPage);
    }

    public function hasAcces() {
        $user = wp_get_current_user();
        $roles = (array) $user->roles;
        foreach ($roles as $role) {
            if (in_array($role, $this->roles)) {
                return true;
            }
        }
        return false;
    }

}
