<?php

namespace expertinvoice\application\models;

use envoqon\db\DbCriteria;
use envoqon\utils\Date;
use envoqon\web\ActiveDataProvider;


class Payment extends \envoqon\db\ActiveRecord {
    
    public $invoice;
    public $customer;

    
    public function tableName() {
        return '{{expertinvoice_payment}}';
    }

    
    public function rules() {
                        return array(
            array('invoiceId,amount,paymentMethod,transactionCode,datePaid', 'required'),
            array('invoiceId, paymentMethod', 'numerical', 'integerOnly' => true),
            array('amount', 'length', 'max' => 12),
            array('dateCreated', 'safe'),
                                    array('id, invoiceId, dateCreated, amount, paymentMethod', 'safe', 'on' => 'search'),
        );
    }

    
    public function getPaymentMethod() {
        return PaymentMethod::model()->findByPk($this->paymentMethod);
    }

    
    public function getInvoice() {
        return Invoice::model()->findByPk($this->invoiceId);
    }

    public function getAttributeLabel($attribute) {
        return __('payment.' . $attribute, 'expert-invoice');
    }

    
    public function search() {
        $q = isset($_GET['q']) ? $_GET['q'] : null;
        $criteria = new DbCriteria;
        
        $criteria->select = "t.*,CONCAT(i.title,' #',i.receiptNumber) AS invoice,c.name AS customer";
        $criteria->join = "LEFT JOIN {{expertinvoice_invoice}} i ON i.id=t.invoiceId"
                . " LEFT JOIN {{expertinvoice_customer}} c ON c.id=i.customerId";

        $criteria->compare('t.id', $q, false, 'OR');
        $criteria->compare('t.invoiceId', $q, false, 'OR');
        $criteria->compare('t.datePaid', $q, true, 'OR');
        $criteria->compare('t.amount', $q, true, 'OR');
        $criteria->compare('t.paymentMethod', $q, false, 'OR');
        $criteria->compare('i.receiptNumber', $q, false, 'OR');
        $criteria->compare('i.title', $q, false, 'OR');

        return new ActiveDataProvider($this, array(
            'criteria' => $criteria,
            'sort' => array(
                'defaultOrder' => 'datePaid DESC',
            ),
        ));
    }

    
    public static function model($className = __CLASS__) {
        return parent::model($className);
    }
    
    protected function beforeSave() {
        if (parent::beforeSave()) {
            if ($this->isNewRecord) {
                $this->dateCreated = Date::format(time(), Date::PATTERN_ISO_8601_DATETIME);
            }
            return true;
        }
        return false;
    }
    
    protected function afterSave() {
        parent::afterSave();
        $invoice = Invoice::model()->findByPk($this->invoiceId);
        if($invoice){
            $invoice->updateStatus();
        }
    }
    
    protected function afterFind() {
        parent::afterFind();
    }
    
    public function fields() {
        $this->invoice = $this->getInvoice()->receiptNumber;
        return array(
            'id', 'invoiceId', 'amount', 'reference', 'paymentMethod', 'datePaid','invoice');
    }
}
