<?php

namespace expertinvoice\application\models;

use envoqon\db\ActiveRecord;
use envoqon\db\DbCriteria;
use envoqon\utils\Date;
use envoqon\utils\WpUtil;
use envoqon\web\ActiveDataProvider;


class Refund extends ActiveRecord {

    public $invoice;
    public $customer;

    
    public function tableName() {
        return '{{expertinvoice_refund}}';
    }

    
    public function rules() {
                        return array(
            array('invoiceId,refundDate', 'required'),
            array('invoiceId, paymentMethod', 'numerical', 'integerOnly' => true),
            array('amount', 'length', 'max' => 12),
            array('reference', 'length', 'max' => 255),
                                    array('id, invoiceId, createdDate, amount, reference, paymentMethod, salesPerson,invoice', 'safe', 'on' => 'search'),
        );
    }
    
    
    public function getPaymentMethod() {
        return PaymentMethod::model()->findByPk($this->paymentMethod);
    }

    
    public function getInvoice() {
        return Invoice::model()->findByPk($this->invoiceId);
    }

    public function getAttributeLabel($attribute) {
        return __('refund.' . $attribute, 'expert-invoice');
    }

    
    public function search() {
        $q = isset($_GET['q']) ? $_GET['q'] : null;
        $criteria = new DbCriteria;
        $criteria->select = "t.*,CONCAT(i.title,' #',i.receiptNumber) AS invoice,c.name AS customer";
        $criteria->join = "LEFT JOIN {{expertinvoice_invoice}} i ON i.id=t.invoiceId"
                . " LEFT JOIN {{expertinvoice_customer}} c ON c.id=i.customerId";

        $criteria->compare('invoiceId', $q, false, 'OR');
        $criteria->compare('refundDate', $q, true, 'OR');
        $criteria->compare('amount', $q, true, 'OR');
        $criteria->compare('reference', $q, true, 'OR');
        $criteria->compare('paymentMethod', $q, false, 'OR');
        $criteria->compare('i.receiptNumber', $q, false, 'OR');
        $criteria->compare('i.title', $q, false, 'OR');

        return new ActiveDataProvider($this, array(
            'criteria' => $criteria,
            'sort' => array(
                'defaultOrder' => 'refundDate DESC',
            ),
        ));
    }

    
    public static function model($className = __CLASS__) {
        return parent::model($className);
    }

    public function fields() {
        $this->invoice = $this->getInvoice()->receiptNumber;
        return array(
            'id', 'invoiceId', 'amount', 'reference', 'paymentMethod', 'refundDate','invoice');
    }

    protected function beforeSave() {
        if (parent::beforeSave()) {
            if ($this->isNewRecord) {
                $this->createdDate = Date::format(time(), Date::PATTERN_ISO_8601_DATETIME);
                $this->salesPerson = WpUtil::getUserId();
            }
            return true;
        }
        return false;
    }
    
    protected function afterSave() {
        parent::afterSave();
        $invoice = Invoice::model()->findByPk($this->invoiceId);
        if($invoice){
            $invoice->updateStatus();
        }
    }

}
