<?php

namespace expertinvoice\application\models;

use envoqon\db\DbCriteria;
use envoqon\web\ActiveDataProvider;


class Tax extends \envoqon\db\ActiveRecord {

    private static $names;
    
    public function tableName() {
        return '{{expertinvoice_tax}}';
    }

    
    public function rules() {
                        return array(
            array('name,rate', 'required'),
            array('name', 'length', 'max' => 255),
            array('rate', 'length', 'max' => 12),
            array('description','safe'),
                                    array('id, name, rate', 'safe', 'on' => 'search'),
        );
    }

    
    public function getProducts() {
        return Product::model()->findAllByAttributes(array('taxId' => $this->getPrimaryKey()));
    }

    
    public function getItem() {
        return InvoiceItem::model()->findByPk($this->id);
    }

    public function getAttributeLabel($attribute) {
        return __('tax.' . $attribute, 'expert-invoice');
    }

    
    public function search() {
        $q = isset($_GET['q']) ? $_GET['q'] : null;
        $criteria = new DbCriteria;

        $criteria->compare('id', $q, false, 'OR');
        $criteria->compare('name', $q, true, 'OR');
        $criteria->compare('rate', $q, true, 'OR');
        $criteria->compare('description', $q, true, 'OR');

        return new ActiveDataProvider($this, array(
            'criteria' => $criteria,
            'sort'=>array(
                'defaultOrder'=>'name'
            )
        ));
    }

    
    public static function model($className = __CLASS__) {
        return parent::model($className);
    }
    
    public function listAll() {
        if (self::$names != null) {
            return self::$names;
        }
        $cat = $this->findAll(array('order' => 'name'));
        self::$names = array();
        foreach ($cat as $t) {
            self::$names[$t->id] = $t->name;
        }
        return self::$names;
    }

    public function getName($id) {
        $n = $this->listAll();
        return isset($n[$id]) ? $n[$id] : '';
    }

    public function fields(){
        return array(
            'id','name','description','rate'
        );
    }
}
