<?php
namespace expertinvoice\application\utils;

use envoqon\web\Html;




class Helper {

    static $ALLOWED_CHARS = '123456789ABCDEFGHIJKLMNPQRSTUVWXYZ';

    static $COLORS = ['#3366CC', '#DC3912', '#FF9900', '#109618', '#990099', '#3B3EAC', '#0099C6', '#DD4477',
        '#66AA00', '#B82E2E', '#316395', '#994499', '#22AA99', '#AAAA11', '#6633CC', '#E67300',
        '#8B0707', '#329262', '#5574A6', '#3B3EAC'];

    public static function getColor($index) {
        $i = $index % count(self::$COLORS);
        return self::$COLORS[$i];
    }
    
    private static $csv = null;

    private static function getCSV() {
        if (self::$csv == null) {
            $file = fopen(__DIR__. '/country-code-to-currency-code-mapping.csv', 'r');
            while (($line = fgetcsv($file)) !== FALSE) {
                                self::$csv[] = $line;
            }
            fclose($file);
        }
        return self::$csv;
    }
    
    public static function getCountryCode($country){
        foreach (self::getCSV() as $value) {
            if ($value[0] == $country) {
                return $value[1];
            }
        }
        return '';
    }

    public static function getCurrency($countryCode) {
        foreach (self::getCSV() as $value) {
            if ($value[1] == $countryCode) {
                return $value[3];
            }
        }
        return '';
    }


    public static function getTmpDir() {
        return get_temp_dir();
    }

    
    public static function lowStockReminder() {
        $stocks = \envoqon\db\QueryBuilder::get()->select('t.quantity,l.name AS location,i.name AS item,i.sku')
                ->from('{{upventory_stock}}', 't')
                ->join('{{upventory_item}} i', 'i.id=t.itemId')
                ->join('{{upventory_location}} l', 'l.id=t.locationId')
                ->andWhere('t.warnQuantity >= t.quantity')
                ->eq('sendEmail', 1)
                ->query();
        $msg = '';
        foreach ($stocks as $stock) {
            $msg .= sprintf("<tr><td>%s - %s</td><td>%s</td><td>%s</td></tr>", $stock['item'], $stock['sku'], $stock['quantity'], $stock['location']);
        }
        if ($msg != '') {
            $msg = "The following items are low on stock <table><thead><tr><th>Name</th><th>Quantity</th><th>Location</th></tr></thead><tbody>" .$msg;
            $msg .=  '</tbody></table>';
            wp_mail(bloginfo('admin_email'), "Low Level Stock", $msg);
        }
    }


    public static function dropDownList($name, $select, $data, $htmlOptions, $showPrompt = false) {
        if ($showPrompt) {
            $htmlOptions['prompt'] = 'All';
        }
        return Html::dropDownList($name, $select, $data, $htmlOptions);
    }

    
    public static function renderFile($_viewFile_, $_data_ = null, $_return_ = false) {
                if (is_array($_data_))
            extract($_data_, EXTR_PREFIX_SAME, 'data');
        else
            $data = $_data_;
        if ($_return_) {
            ob_start();
            ob_implicit_flush(false);
            require($_viewFile_);
            return ob_get_clean();
        }
        else
            require($_viewFile_);
    }
}
