<?php

namespace expertinvoice\application\utils;

use envoqon\db\QueryBuilder;
use envoqon\utils\Date;
use envoqon\utils\WpUtil;
use envoqon\web\Html;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;
use expertinvoice\application\models\Product;


class ReportHelper {

    const REPORT_FIGURES = 0;
    const REPORT_DATA = 1;
    const REPORT_CHART = 2;
    const REPORT_ALL = 3;
    const GRAPH_LINE = 'line';
    const GRAPH_AREASPLINE = 'areaspline';

    private static function echoEmptyTd($number, $tag = 'td') {
        $empty = array();
        for ($index = 0; $index < $number; $index++) {
            echo Html::tag($tag, $empty, '&nbsp;');
        }
    }

    public static function inventory($attributes, $title) {
        $query = QueryBuilder::get()->from('{{expertinvoice_product}}', 's')
                ->select('name,sku,description,salePrice,quantity,isManaged');
        $rows = $query->query();
        $empty = array();
        $_span = 8;
        ob_start();
        echo '<tr>';
        echo '<th>' . __('report.Product', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.sku', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.description', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.salePrice', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.quantity', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.Value', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        $overaltotal = 0;
        $quantityTotal = 0;
        $lineTotal = 0;
        $option = Option::getAppOption();
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, $row['name']);
            echo Html::tag('td', $empty, $row['sku']);
            echo Html::tag('td', $empty, $row['description']);
            echo Html::tag('td', $empty, $option->formatMoney($row['salePrice']));
            if ($row['isManaged'] == '1') {
                echo Html::tag('td', $empty, $row['quantity']);
            }
            else {
                echo Html::tag('td', $empty, '&nbsp;');
            }
            $lineTotal = $row['quantity'] * $row['salePrice'];
            if ($row['isManaged'] == '1') {
                echo Html::tag('td', $empty, $option->formatMoney($lineTotal));
            }
            else {
                echo Html::tag('td', $empty, '&nbsp;');
            }
            $overaltotal += $lineTotal;
            $quantityTotal += $row['quantity'];
            echo '</tr>';
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        self::echoEmptyTd(3, 'th');
        echo Html::tag('th', array('class' => 'total'), $option->formatInteger($quantityTotal));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        $summary = array();
        $summary['Count of Products'] = $option->formatInteger($quantityTotal);
        $summary['Total Value'] = $option->formatMoney($overaltotal);
        self::echoSummary($title, $summary, $_span, $attributes);
        echo $buffer;
    }

    public static function lowLevelStock($attributes, $title) {
        $query = QueryBuilder::get()->from('{{expertinvoice_product}}')
                ->select('name,sku,description,salePrice,quantity,warnQuantity')
                ->eq('isManaged', 1)
                ->andWhere('warnQuantity >= quantity');
        $rows = $query->query();
        $empty = array();
        $_span = 6;
        ob_start();
        echo '<tr>';
        echo '<th>' . __('product.name', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.sku', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.description', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.quantity', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.warnQuantity', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, $row['name']);
            echo Html::tag('td', $empty, $row['sku']);
            echo Html::tag('td', $empty, $row['description']);
            echo Html::tag('td', $empty, Option::getAppOption()->formatInteger($row['quantity']));
            echo Html::tag('td', $empty, $row['warnQuantity']);
            echo '</tr>';
        }
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
                ), $_span, $attributes);
        echo $buffer;
    }

    public static function invoice($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('t.invoicedDate,t.receiptNumber,t.amountDue,t.totalTax,c.name AS customer,u.display_name AS user,SUM(p.amount) AS payment,SUM(r.amount) AS refund')
                ->from('{{expertinvoice_invoice}}', 't')
                ->leftJoin(WpUtil::getTableName('users') . ' u', 'u.ID=t.salesPerson')
                ->leftJoin('{{expertinvoice_payment}} p', 'p.invoiceId=t.id')
                ->leftJoin('{{expertinvoice_refund}} r', 'r.invoiceId=t.id')
                ->leftJoin('{{expertinvoice_customer}} c', 'c.id=t.customerId')
                ->between('t.invoicedDate', $attributes['start'], $endDate)
                ->eq('t.type', Invoice::TYPE_INVOICE)
                ->eq('t.status', $attributes['status'])
                ->group('t.id')
                ->order('t.invoicedDate DESC');
        $rows = $query->query();
        $empty = array();
        $_span = 9;
        $overaltotal = 0;
        $overalPayments = 0;
        $overalRefund = 0;
        $numberOfTransactions = 0;
        $overaltaxes = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('invoice.invoicedDate', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.receiptNumber', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.salesPerson', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.customerId', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.InvoiceAmount', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.Taxes', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.InvoicePayment', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.InvoiceRefund', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.NetAmount', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, Date::formatDateTime($row['invoicedDate'], $option->dateFormat));
            echo Html::tag('td', $empty, $row['receiptNumber']);
            echo Html::tag('td', $empty, $row['user']);
            echo Html::tag('td', $empty, $row['customer']);
            echo Html::tag('td', $empty, $option->formatMoney($row['amountDue']));
            echo Html::tag('td', $empty, $option->formatMoney($row['totalTax']));
            echo Html::tag('td', $empty, $option->formatMoney($row['payment']));
            echo Html::tag('td', $empty, $option->formatMoney($row['refund']));
            echo Html::tag('td', $empty, $option->formatMoney($row['amountDue'] + $row['refund'] - $row['payment']));
            $overaltotal += $row['amountDue'];
            $overalPayments += $row['payment'];
            $overalRefund += $row['refund'];
            $overaltaxes += $row['totalTax'];
            echo '</tr>';
            ++$numberOfTransactions;
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        self::echoEmptyTd(3, 'th');
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal)); 
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltaxes));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overalPayments));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overalRefund));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal + $overalRefund - $overalPayments));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
            'Number of Transactions' => $option->formatInteger($numberOfTransactions),
            'Total Amount' => $option->formatMoney($overaltotal),
            'Total Taxes' => $option->formatMoney($overaltaxes),
            'Total Payments' => $option->formatMoney($overalPayments),
            'Total Refund' => $option->formatMoney($overalRefund),
            'Payment Due' => $option->formatMoney($overaltotal + $overalRefund - $overalPayments)
                ), $_span, $attributes);
        echo $buffer;
    }

    public static function quotes($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('t.invoicedDate,t.quoteNumber,t.amountDue,u.display_name AS user')
                ->from('{{expertinvoice_invoice}}', 't')
                ->leftJoin(WpUtil::getTableName('users') . ' u', 'u.ID=t.salesPerson')
                ->between('t.invoicedDate', $attributes['start'], $endDate)
                ->eq('t.type', Invoice::TYPE_QUOTE)
                ->group('t.id')
                ->order('t.invoicedDate DESC');
        $rows = $query->query();
        $empty = array();
        $_span = 4;
        $overaltotal = 0;
        $overalTaxes = 0;
        $numberOfTransactions = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('quote.invoicedDate', 'expert-invoice') . '</th>';
        echo '<th>' . __('quote.quoteNumber', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.salesPerson', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.amountDue', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, Date::formatDateTime($row['invoicedDate'], $option->dateFormat));
            echo Html::tag('td', $empty, $row['quoteNumber']);
            echo Html::tag('td', $empty, $row['user']);
            echo Html::tag('td', $empty, $option->formatMoney($row['amountDue']));
            $overaltotal += $row['amountDue'];
            $overalTaxes += $row['totalTax'];
            echo '</tr>';
            ++$numberOfTransactions;
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        self::echoEmptyTd(1, 'th');
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overalTaxes));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
            'Number of Transactions' => $option->formatInteger($numberOfTransactions),
            'Total Amount' => $option->formatMoney($overaltotal),
            'Total Taxes' => $option->formatMoney($overalTaxes)
                ), $_span, $attributes);
        echo $buffer;
    }

    public static function itemSales($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('SUM(t.quantity) AS qty, SUM(t.quantity * (t.salePrice - t.discount)) AS totalAmount,i.name, i.sku')
                ->from('{{expertinvoice_invoice_item}}', 't')
                ->leftJoin('{{expertinvoice_product}} i', 'i.id=t.productId')
                ->join('{{expertinvoice_invoice}} s', 's.id=t.invoiceId')
                ->between('s.invoicedDate', $attributes['start'], $endDate)
                ->eq('s.type', Invoice::TYPE_INVOICE)
                ->group('t.productId');
        if (!empty($attributes['productId'])) {
            $query->eq('t.productId', $attributes['productId']);
        }
        $rows = $query->query();
        $empty = array();
        $_span = 4;
        $overaltotal = 0;
        $numberOfTransactions = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('product.name', 'expert-invoice') . '</th>';
        echo '<th>' . __('product.sku', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.Quantity', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.Value', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, $row['name']);
            echo Html::tag('td', $empty, $row['sku']);
            echo Html::tag('td', $empty, $row['qty']);
            echo Html::tag('td', $empty, $option->formatInteger($row['totalAmount']));
            $overaltotal += $row['totalAmount'];
            echo '</tr>';
            ++$numberOfTransactions;
        }
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        $summary = array(
        );
        if (!empty($attributes['productId'])) {
            $item = Product::model()->findByPk($attributes['productId']);
            if ($item != null) {
                $summary['Product'] = $item->name . ' (' . $item->sku . ')';
            }
        }
        $summary['Number of Products'] = $option->formatInteger($numberOfTransactions);
        $summary['Total Sales'] = $option->formatMoney($overaltotal);
        self::echoSummary($title, $summary, $_span, $attributes);
        echo $buffer;
    }

    public static function salesTax($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('SUM(t.tax) AS taxes,i.name')
                ->from('{{expertinvoice_invoice_item}}', 't')
                ->leftJoin('{{expertinvoice_tax}} i', 'i.id=t.taxId')
                ->leftJoin('{{expertinvoice_invoice}} s', 's.id=t.invoiceId')
                ->eq('s.status', Invoice::STATUS_PAID)
                ->between('s.invoicedDate', $attributes['start'], $endDate)
                ->neq('t.taxId', 1)
                ->group('t.taxId');

        $rows = $query->query();
        $empty = array();
        $_span = 2;
        $overaltotal = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('tax.name', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.Amount', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, $row['name']);
            echo Html::tag('td', $empty, $option->formatMoney($row['taxes']));
            echo '</tr>';
            $overaltotal += $row['taxes'];
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
            'Total Taxes' => $option->formatMoney($overaltotal),
                ), $_span, $attributes);
        echo $buffer;
    }

    public static function refunds($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('t.*,i.receiptNumber,c.name AS customer,u.display_name AS user')
                ->from('{{expertinvoice_refund}}', 't')
                ->leftJoin(WpUtil::getTableName('users') . ' u', 'u.ID=t.salesPerson')
                ->leftJoin('{{expertinvoice_invoice}} i', 'i.id=t.invoiceId')
                ->leftJoin('{{expertinvoice_customer}} c', 'c.id=i.customerId')
                ->between('t.refundDate', $attributes['start'], $endDate)
                ->eq('t.paymentMethod', $attributes['paymentMethod'])
                ->order('t.refundDate DESC');
        $rows = $query->query();
        $empty = array();
        $_span = 6;
        $overaltotal = 0;
        $numberOfTransactions = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('refund.refundDate', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.receiptNumber', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.customerId', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.salesPerson', 'expert-invoice') . '</th>';
        echo '<th>' . __('refund.amount', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, Date::formatDateTime($row['refundDate'], $option->dateFormat));
            echo Html::tag('td', $empty, $row['receiptNumber']);
            echo Html::tag('td', $empty, $row['customer']);
            echo Html::tag('td', $empty, $row['user']);
            echo Html::tag('td', $empty, $option->formatMoney($row['amount']));
            echo '</tr>';
            $overaltotal += $row['amount'];
            ++$numberOfTransactions;
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        self::echoEmptyTd(3, 'th');
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
            'Number of Transactions' => $option->formatInteger($numberOfTransactions),
            'Total Amount' => $option->formatMoney($overaltotal),
                ), $_span, $attributes);
        echo $buffer;
    }

    public static function payments($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('t.*,i.receiptNumber,c.name AS customer')
                ->from('{{expertinvoice_payment}}', 't')
                ->leftJoin('{{expertinvoice_invoice}} i', 'i.id=t.invoiceId')
                ->leftJoin('{{expertinvoice_customer}} c', 'c.id=i.customerId')
                ->between('t.datePaid', $attributes['start'], $endDate)
                ->eq('t.paymentMethod', $attributes['paymentMethod'])
                ->order('t.datePaid DESC');
        $rows = $query->query();
        $empty = array();
        $_span = 5;
        $overaltotal = 0;
        $numberOfTransactions = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('payment.datePaid', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.receiptNumber', 'expert-invoice') . '</th>';
        echo '<th>' . __('invoice.customerId', 'expert-invoice') . '</th>';
        echo '<th>' . __('payment.amount', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, Date::formatDateTime($row['datePaid'], $option->dateFormat));
            echo Html::tag('td', $empty, $row['receiptNumber']);
            echo Html::tag('td', $empty, $row['customer']);
            echo Html::tag('td', $empty, $option->formatMoney($row['amount']));
            echo '</tr>';
            $overaltotal += $row['amount'];
            ++$numberOfTransactions;
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        self::echoEmptyTd(2, 'th');
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
            'Number of Transactions' => $option->formatInteger($numberOfTransactions),
            'Total Amount' => $option->formatMoney($overaltotal),
                ), $_span, $attributes);
        echo $buffer;
    }

    public static function customerSales($attributes, $title) {
        $endDate = self::getEndDate($attributes);
        $query = QueryBuilder::get()
                ->select('t.amountDue,SUM(p.amount) AS payment,SUM(r.amount) AS refund,c.name AS customer')
                ->from('{{expertinvoice_invoice}}', 't')
                ->leftJoin('{{expertinvoice_payment}} p', 'p.invoiceId=t.id')
                ->leftJoin('{{expertinvoice_refund}} r', 'r.invoiceId=t.id')
                ->leftJoin('{{expertinvoice_customer}} c', 'c.id=t.customerId')
                ->between('t.invoicedDate', $attributes['start'], $endDate)
                ->eq('t.type', Invoice::TYPE_INVOICE)
                ->eq('t.customerId', $attributes['customerId'])
                ->group('t.customerId')
                ->order('t.invoicedDate DESC');
        $rows = $query->query();
        $empty = array();
        $_span = 4;
        $overaltotal = 0;
        $overalTaxes = 0;
        $overalRefund = 0;
        $numberOfTransactions = 0;
        $option = Option::getAppOption();
        ob_start();
        echo '<tr>';
        echo '<th>' . __('invoice.customerId', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.InvoiceAmount', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.InvoicePayment', 'expert-invoice') . '</th>';
        echo '<th>' . __('report.InvoiceRefund', 'expert-invoice') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($rows as $row) {
            echo '<tr>';
            echo Html::tag('td', $empty, $row['customer']);
            echo Html::tag('td', $empty, $option->formatMoney($row['amountDue']));
            echo Html::tag('td', $empty, $option->formatMoney($row['payment']));
            echo Html::tag('td', $empty, $option->formatMoney($row['refund']));
            $overaltotal += $row['amountDue'];
            $overalTaxes += $row['payment'];
            $overalRefund += $row['refund'];
            echo '</tr>';
        }
        echo '<tr>';
        echo '<th class="total-label">Total</th>';
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overaltotal));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overalTaxes));
        echo Html::tag('th', array('class' => 'total'), $option->formatMoney($overalRefund));
        echo '</tr>';
        echo '</tbody>';
        echo '</table>';
        $buffer = ob_get_clean();
        self::echoSummary($title, array(
            'Total Invoice Amount' => $option->formatMoney($overaltotal),
            'Total Payments' => $option->formatMoney($overalTaxes),
            'Total Refund' => $option->formatMoney($overalRefund)
                ), $_span, $attributes);
        echo $buffer;
    }

    private static function echoSummary($title, $summary, $numCols, $attributes) {
        if (empty($summary)) {
            $summaryCol = $numCols - $title;
            echo '<table class="simple" id="report-table">';
            echo '<thead>';
            echo '<tr>';
            echo '<th colspan="' . $numCols . '" class="report-title" style="border-right:none">' . self::getFullTitle($title, $attributes) . '</th>';
            echo '</tr>';
        }
        else {
            $titleCol = abs(($numCols + 1) / 2);
            $summaryCol = $numCols - $title;
            echo '<table class="simple" id="report-table">';
            echo '<thead>';
            echo '<tr>';
            echo '<th colspan="' . $titleCol . '" class="report-title" style="border-right:none">' . self::getFullTitle($title, $attributes) . '</th>';
            echo '<th colspan="' . $summaryCol . '" style="border-left:none">';
            foreach ($summary as $key => $value) {
                echo '<p>';
                echo '<span style="display:inline-block;width: 50%;">' . $key . ':</span>';
                echo '<span style="display:inline-block">' . $value . '</span>';
                echo '</p>';
            }
            echo '</th>';
            echo '</tr>';
        }
    }

    private static function getFullTitle($title, $attributes) {
        $label = '';
        if (isset($attributes['start']) && !empty($attributes['start'])) {
            $label .= '<small><i>From</i> ' . Date::formatDateTime($attributes['start'], Option::getAppOption()->dateFormat) . '</small>';
        }
        if (isset($attributes['end']) && !empty($attributes['end'])) {
            $label .= ' <small>- <i>Ending</i> ' . Date::formatDateTime($attributes['end'], Option::getAppOption()->dateFormat) . '</small>';
        }
        return $title . ($label ? '<br/>' . $label : '');
    }

    private static function getEndDate($attributes) {
        return !empty($attributes['end']) ? $attributes['end'] . ' 23:59:59' : '';
    }

}
