<?php

use envoqon\utils\Date;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;

$option = Option::getAppOption();
?>
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <meta name="viewport" content="width=device-width,initial-scale=1" />
        <link rel="stylesheet" href="<?php echo ExpertInvoicePlugin::$app->getPluginUrl() . '/css/invoice.css'; ?>" type="text/css" media="all" />
    </head>
    <body>
        <div id="receipt-template">
            <div class="header">
                <div>
                <?php if ($option->hasLogo()): ?>
                    <img class="logo" src="<?php echo $option->getLogoLink(); ?>"/>
                <?php endif; ?>
                </div>
                <h1 class="align-center">Invoice</h1>
                <table>
                    <tbody>
                        <tr>
                            <td>
                                <h3 class="compname">
                                    <?php echo $option->name; ?>
                                </h3>
                                    <?php echo nl2br($option->contactDetails); ?>
                            </td>
                            <td class="align-right">
                                Invoice No.: <?php echo $model->receiptNumber; ?>
                                <p>
                                    Date: <?php echo Date::formatDateTime($model->invoicedDate, $option->dateFormat); ?>
                                </p>
                                <?php if($model->getStatus() != Invoice::STATUS_PAID):?>
                                    <p>
                                        <?php echo $option->invoiceDueDate; ?>: <?php echo Date::formatDateTime($model->dueDate, $option->dateFormat); ?>
                                    </p>
                                <?php endif;?>
                            </td>
                        </tr>
                    </tbody>
                </table> 
                <p class="bold bill-to">Bill To:</p>
                <p class="line-item"><?php echo $customer->name; ?></p>
                <p class="line-item"><?php echo nl2br($customer->shippingAddress); ?></p>
            </div>
            <div class="main">
                <table>
                    <thead>
                        <tr>
                            <th class="">Product Name</th>
                            <th class="align-right"><?php echo $option->unitPriceColumn; ?></th>
                            <th class="align-right"><?php echo $option->quantityColumn; ?></th>
                            <th class="align-right"><?php echo $option->lineTotalColumn; ?></th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php foreach ($items as $item) : ?>
                            <tr class="">
                                <td>
                                    <?php echo $item->name; ?>
                                </td>
                                <td class="align-right">
                                    <?php echo $option->formatMoney($item->salePrice - $item->discount); ?>
                                </td>
                                <td class="align-right">
                                    <?php echo $item->quantity; ?>
                                </td>
                                <td class="align-right">
                                    <?php echo $option->formatMoney(($item->salePrice - $item->discount) * $item->quantity); ?>
                                </td>
                            </tr>
                        <?php endforeach; ?>
                        <tr>
                            <td colspan="4">&nbsp;</td>
                        </tr>
                        <tr>
                            <td colspan="3" class="align-right">
                                <?php echo $option->subTotal; ?>:
                            </td>
                            <td class="align-right">
                                <?php echo $option->formatMoney($model->amountDue + $model->discount - $model->totalTax); ?>
                            </td>
                        </tr>
                        <?php if (floatval($model->discount) > 0): ?>
                            <tr>
                                <td colspan="3" class="align-right">
                                    <?php echo $option->discountColumn; ?>:
                                </td>
                                <td class="align-right">
                                    <?php echo $option->formatMoney($model->discount); ?>
                                </td>
                            </tr>
                        <?php endif; ?>
                        <?php if ($model->totalTax > 0): ?>
                            <tr>
                                <td colspan="3" class="align-right">
                                    Tax:
                                </td>
                                <td class="align-right">
                                    <?php echo $option->formatMoney($model->totalTax); ?>
                                </td>
                            </tr>
                        <?php endif; ?>
                        <?php if ($model->amountDue > 0): ?>
                            <tr>
                                <td colspan="3" class="align-right">
                                    <?php echo $option->totalAmount; ?>:
                                </td>
                                <td class="align-right">
                                    <?php echo $option->formatMoney($model->amountDue); ?>
                                </td>
                            </tr>
                        <?php endif; ?>
                        <?php if ($model->totalPaid > 0): ?>
                            <tr>
                                <td colspan="3" class="align-right">
                                    <?php echo $option->amountTendered; ?>:
                                </td>
                                <td class="align-right">
                                    <?php echo $option->formatMoney($model->totalPaid); ?>
                                </td>
                            </tr>
                        <?php endif; ?>
                    </tbody>
                </table>
                <?php if ($model->comment): ?>
                    <p>
                        <?php echo $model->comment; ?>
                    </p>
                <?php endif; ?>
            </div>
        </div>
    </body>
</html>