<?php

use envoqon\utils\Date;
use expertinvoice\application\models\Customer;
use expertinvoice\application\models\Invoice;
use expertinvoice\application\models\Option;



?>
<div class="wrap" id="inner-content">
    <div class="toolbar">
        <h1 class="wp-heading-inline"><?php echo $this->pageTitle; ?></h1>
        <a href="<?php echo $this->createUrl('expert-invoice-invoice', 'create'); ?>" class="page-title-action">Add Invoice</a>
    </div>
    <?php
    $today = Date::format(time(), Date::PATTERN_ISO_8601_DATE);
    $this->widget('envoqon\widgets\GridView', array(
        'id' => 'invoice-grid',
        'dataProvider' => $model->invoiceSearch(),
        'selectableRows' => 1,
        'columns' => array(
            array(
                'class' => 'envoqon\widgets\PrimaryColumn',
                'header' => __('invoice.title', 'expert-invoice'),
                'template' => '{update}{duplicate}{addPayment}{addRefund}{view}{print}{email}{delete}',
                'labelExpression' => '$data["title"]',
                'urlExpression' => function($data) {
                    return ExpertInvoicePlugin::$app->controller->createUrl("expert-invoice-invoice", "update", array("id" => $data["id"]));
                },
                'disabledExpression' => '$data->getStatus() == ' . Invoice::STATUS_PAID,
                'buttons' => array(
                    'addPayment' => array(
                        'label' => 'Add Payment',
                        'visible' => '$data->getStatus() != ' . Invoice::STATUS_PAID,
                        'click' => 'function(e){
                            e.preventDefault();
                            var id = jQuery(this).closest("tr").data("key");
                            global.controller.addPayment(id);
                    }',
                    ),
                    'addRefund' => array(
                        'label' => 'Refund',
                        'visible' => '$data->getStatus() != ' . Invoice::STATUS_UNPAID,
                        'click' => 'function(e){
                            e.preventDefault();
                            var id = jQuery(this).closest("tr").data("key");
                            global.controller.addRefund(id);
                    }',
                    ),
                    'email' => array(
                        'label' => 'Email',
                        'click' => 'function(e){
                            e.preventDefault();
                            var id = jQuery(this).closest("tr").data("key");
                            global.controller.email(id);
                    }',
                    ),
                    'print' => array(
                        'label' => 'Print',
                        'click' => 'function(e){
                            e.preventDefault();
                            var id = jQuery(this).closest("tr").data("key");
                            global.controller.printWindow("admin-ajax.php?action=' . ExpertInvoicePlugin::$app->getAjaxAction() . '&r=invoice/print&id="+id);
                    }',
                    ),
                    'duplicate' => array(
                        'label' => 'Duplicate',
                        'url' => 'ExpertInvoicePlugin::$app->controller->createUrl("expert-invoice-invoice","duplicate",array("id"=>$data["id"]))'
                    ),
                    'update' => array(
                        'class' => 'envoqon\widgets\UpdateActionLink',
                        'visible' => '$data->getStatus() == ' . Invoice::STATUS_UNPAID,
                        'url' => 'ExpertInvoicePlugin::$app->controller->createUrl("expert-invoice-invoice","update",array("id"=>$data["id"]))'
                    ),
                    'delete' => array(
                        'class' => 'envoqon\widgets\DeleteActionLink',
                        'afterDelete' => 'gridViewAfterDelete()',
                        'visible' => '$data->getStatus() == ' . Invoice::STATUS_UNPAID,
                        'url' => 'ExpertInvoicePlugin::$app->controller->createAjaxUrl("invoice/delete",array("id"=>$data["id"]))'
                    ),
                    'view' => array(
                        'label' => 'View',
                        'url' => '$data->getViewUrl()',
                        'options'=>array(
                            'target'=>'_blank'
                        )
                    ),
                ),
            ),
            array(
                'header' => __('invoice.receiptNumber', 'expert-invoice'),
                'value' => function($data, $row) {
                    return $data['receiptNumber'];
                },
            ),
            array(
                'name' => 'customerId',
                'header' => __('invoice.customerId', 'expert-invoice'),
                'value' => function($data, $row) {
                    return $data['customer'];
                },
            ),
            array(
                'header' => __('invoice.invoicedDate', 'expert-invoice'),
                'value' => function($data, $row) {
                    return Date::formatDateTime($data['invoicedDate'], Option::getAppOption()->dateFormat);
                },
            ),
            array(
                'header' => __('invoice.amountDue', 'expert-invoice'),
                'value' => function($data, $row) {
                    return Option::getAppOption()->formatMoney($data['amountDue']);
                },
            ),
            array(
                'header' => __('invoice.totalPaid', 'expert-invoice'),
                'value' => function($data, $row) {
                    return Option::getAppOption()->formatMoney($data['totalPaid']);
                },
            ),
            array(
                'header' => __('invoice.status', 'expert-invoice'),
                'value' => function($data, $row) {
                    return $data->getStatusLabel();
                },
            ),
            array(
                'header' => __('invoice.dueDate', 'expert-invoice'),
                'type'=>'raw',
                'value' => function($data, $row) use($today) {
                    if($data->getStatus() ==  Invoice::STATUS_PAID){
                        return '';
                    }
                    if($data['dueDate'] < $today){
                        return '<code>Overdue</code>';
                    }
                    return Date::formatDateTime($data['dueDate'], Option::getAppOption()->dateFormat);
                },
            ),
        ),
    ));
    ?>
</div>
<?php
$this->renderPartial('_payment');
$this->renderPartial('_refund');
?>
<script type="text/javascript">
    jQuery(function () {
        global.controller = new InvoiceController();
        ko.applyBindings({
            controller: global.controller
        }, document.getElementById('inner-content'));
    });
</script>