<?php

namespace envoqon;

use envoqon\base\Component;
use envoqon\db\DbConnection;
use envoqon\db\DbMigration;
use envoqon\logging\Logger;
use envoqon\utils\ChoiceFormat;
use envoqon\utils\Formatter;
use envoqon\web\Controller;
use envoqon\web\HttpException;
use envoqon\web\HttpSession;
use envoqon\web\WebUser;
use Exception;
use ReflectionClass;

defined('ENVOQON_DEBUG') or define('ENVOQON_DEBUG', WP_DEBUG);
defined('ENVOQON_TRACE_LEVEL') or define('ENVOQON_TRACE_LEVEL', 0);
defined('ENVOQON_CURRENT_APP') or define('ENVOQON_CURRENT_APP', '');


class App extends Component {

    private $_viewPath;
    private $_basePath;
    private $_pluginUrl;
    private $frameworkPath;
    private $_name;
    private $options = array();
    private $components = array();
    private $_controller;
    public $charset = 'UTF-8';
    public $_format;
    private $_ajaxAction;
    private $_requestUri;
    private static $app; 
    
    private static $db;

    
    private function __construct() {
        $this->frameworkPath = dirname(__FILE__);
    }

    
    public static function init($config) {
        $app = new App();
        $app->configure($config);
        self::$app = $app;         add_action('admin_notices', array($app, 'echoNotice'));
        add_action('init', array($app, 'startSession'));
        add_action('wp_logout', array($app, 'destroySession'));
        $app->options = get_option($app->getName(), array());
        return $app;
    }

    
    public static function instance() {
        return self::$app;
    }

    
    public function startSession() {
        $this->getSession()->open();
    }

    
    public function destroySession() {
        $this->getSession()->destroy();
    }

    
    public function echoNotice() {
        if (!wp_doing_ajax()) {
            $this->getUser()->echoNotice();
        }
    }

    
    public function configure($config) {
        if (is_array($config)) {
            foreach ($config as $key => $value) {
                $this->$key = $value;
            }
        }
    }

    
    public function getSession() {
        return $this->getComponent('session', array('class' => 'envoqon\web\HttpSession'));
    }

    
    public function getUser() {
        return $this->getComponent('user', array('class' => 'envoqon\web\WebUser'));
    }

    
    public function getComponent($id, $config = array()) {
        if (isset($this->components[$id])) {
            return $this->components[$id];
        }
        else {
            $obj = $this->createComponent($config);
            $obj->init();
            $this->components[$id] = $obj;
            return $obj;
        }
    }

    
    public function registerScriptFile($id, $url, $depends = array()) {
        wp_enqueue_script($id, $url, $depends, true);
    }

    
    public function registerCssFile($id, $url, $depends = array()) {
        wp_enqueue_style($id, $url, $depends);
    }

    
    public function registerScript($id, $script) {
        $final = '<script>jQuery(document).ready(function($) {' . $script . '})</script>';
        echo $final;
    }

    
    public function registerCss($id, $css) {
        $final = '<style>' . $css . '</style>';
        echo $final;
    }

    
    public static function createComponent($config) {
        if (is_string($config)) {
            $type = $config;
            $config = array();
        }
        elseif (isset($config['class'])) {
            $type = $config['class'];
            unset($config['class']);
        }
        else
            throw new Exception(App::t('app', 'Object configuration must be an array containing a "class" element.'));

        if (($n = func_num_args()) > 1) {
            $args = func_get_args();
            if ($n === 2)
                $object = new $type($args[1]);
            elseif ($n === 3)
                $object = new $type($args[1], $args[2]);
            elseif ($n === 4)
                $object = new $type($args[1], $args[2], $args[3]);
            else {
                unset($args[0]);
                $class = new ReflectionClass($type);
                                                $object = call_user_func_array(array($class, 'newInstance'), $args);
            }
        }
        else
            $object = new $type;

        foreach ($config as $key => $value)
            $object->$key = $value;

        return $object;
    }

    
    public function getOption($name, $default = null) {
        if (isset($this->options[$name])) {
            return $this->options[$name];
        }
        return $default;
    }

    
    public function setOption($name, $value) {
        $this->options[$name] = $value;
        if (!update_option($this->getName(), $this->options)) {
            add_option($this->getName(), $this->options);
        }
    }

    
    public function getFormat() {
        if (!$this->_format) {
            $this->_format = new Formatter();
        }
        return $this->_format;
    }

    
    public function runAdmin() {
        $controllerName = isset($_GET['page']) ? $_GET['page'] : $this->getName();
        $controllerId = str_replace($this->getName() . '-', '', $controllerName);
        $parts = explode('-',$controllerId);
        if(count($parts) > 1){
            $controllerId = $parts[0] . join(array_map('ucfirst', array_splice($parts, 1)));
        }
        else{
            $controllerId = $parts[0];
        }
        $actionId = isset($_GET['action']) ? $_GET['action'] : 'index';
        $this->runImp($controllerId, $actionId);
    }

    
    public function runAjax() {
        $route = isset($_GET['r']) ? $_GET['r'] : 'ajax';
        $routeArrays = explode('/', $route);
        $actionId = isset($routeArrays[1]) ? $routeArrays[1] : 'index';
        $this->runImp($routeArrays[0], $actionId);
        $this->end();
    }

    
    public function runFront() {
        $route = isset($_GET['r']) ? $_GET['r'] : 'ajax';
        $routeArrays = explode('/', $route);
        $actionId = isset($routeArrays[1]) ? $routeArrays[1] : 'index';
        $this->runImp(join(array_map('ucfirst', explode('-', $routeArrays[0]))), $actionId);
        $this->end();
    }

    private function runImp($controllerId, $actionId) {
        self::$app = $this;
        $className = ucfirst($controllerId) . 'Controller';
        $basePath = $this->getBasePath() . DIRECTORY_SEPARATOR . 'controllers';
        $classFile = $basePath . DIRECTORY_SEPARATOR . $className . '.php';
        if (is_file($classFile)) {
            if (!class_exists($className, false)) {
                require($classFile);
            }
            if (class_exists($className, false) && is_subclass_of($className, '\envoqon\web\Controller')) {
                $controller = new $className($controllerId);
                $this->_controller = $controller;
                $controller->run($actionId);
                return;
            }
        }
        throw new HttpException("Page not found", 404);
    }

    
    public function getController() {
        return $this->_controller;
    }

    
    public function migrateUp() {
        $lastMigrationTime = $this->getOption('lastMigrationTime', '0');
        $migrations = $this->getNewMigration($lastMigrationTime);
        foreach ($migrations as $migration) {
            if ($this->migrateUpImp($migration) === false) {
                $this->getUser()->setFlash(WebUser::NOTICE_ERROR, $this->getName() . ': There was an error creating tables. The plugin might not work well');
                return false;
            }
        }
        $this->setOption('lastMigrationTime', gmdate('ymd_His'));
        return true;
    }

    
    public function migrateDown() {
        $migrations = $this->getNewMigration('0');
        foreach ($migrations as $migration) {
            if ($this->migrateDownImp($migration) === false) {
                $this->getUser()->setFlash(WebUser::NOTICE_ERROR, $this->getName() . ': There was error deleting the plugins tables. Manually remove the tables.');
                return false;
            }
        }
        $this->setOption('lastMigrationTime', 0);
        return true;
    }

    
    public function deleteOptions() {
        delete_option($this->name);
        $this->options = array();
    }

    private function migrateUpImp($class) {
        $migration = $this->instantiateMigration($class);
        $migration->up();
    }

    private function migrateDownImp($class) {
        $migration = $this->instantiateMigration($class);
        $migration->down();
    }

    private function getMigrationPath() {
        return $this->getBasePath() . DIRECTORY_SEPARATOR . 'migrations';
    }

    private function instantiateMigration($class) {
        $file = $this->getMigrationPath() . DIRECTORY_SEPARATOR . $class . '.php';
        require_once($file);
        $migration = new $class;
        $migration->setDbConnection($this->getDb());
        return $migration;
    }

    private function getNewMigration($lastMigrationTime) {
        $migrations = array();
        $handle = opendir($this->getBasePath() . DIRECTORY_SEPARATOR . 'migrations');
        while (($file = readdir($handle)) !== false) {
            if ($file === '.' || $file === '..')
                continue;
            $path = $this->getMigrationPath() . DIRECTORY_SEPARATOR . $file;
            if (preg_match('/^(m(\d{6}_\d{6})_.*?)\.php$/', $file, $matches) && is_file($path) && ($matches[2]) > $lastMigrationTime) {
                $migrations[] = $matches[1];
            }
        }
        closedir($handle);
        sort($migrations);
        return $migrations;
    }

    
    public static function getDb() {
        if (self::$db == null) {
            self::$db = new DbConnection();
            self::$db->init();
        }
        return self::$db;
    }

    
    public function getName() {
        return $this->_name;
    }

    
    public function setName($value) {
        $this->_name = $value;
    }

    
    public function getAjaxAction() {
        return $this->_ajaxAction;
    }

    
    public function setAjaxAction($value) {
        $this->_ajaxAction = $value;
    }

    
    public function getViewPath() {
        if ($this->_viewPath !== null)
            return $this->_viewPath;
        else
            return $this->_viewPath = $this->getBasePath() . DIRECTORY_SEPARATOR . 'views';
    }

    
    public function setViewPath($path) {
        if (($this->_viewPath = realpath($path)) === false || !is_dir($this->_viewPath)) {
            throw new Exception(App::t('app', 'The view path "{path}" is not a valid directory.', array('{path}' => $path)));
        }
    }

    
    public function getBaseUrl() {
        return $this->getPluginUrl() . '/' . $this->getName();
    }

    
    public function getAdminBaseUrl($absolute = false) {
        return admin_url('admin.php');
    }

    
    public function setBasePath($path) {
        if (($this->_basePath = realpath($path)) === false || !is_dir($this->_basePath)) {
            throw new Exception(App::t('app', 'Application base path "{path}" is not a valid directory.', array('{path}' => $path)));
        }
    }

    
    public function getBasePath() {
        return rtrim($this->_basePath, DIRECTORY_SEPARATOR);
    }

    
    public function getRequestUri() {
        if ($this->_requestUri === null) {
            if (isset($_SERVER['HTTP_X_REWRITE_URL']))                 $this->_requestUri = $_SERVER['HTTP_X_REWRITE_URL'];
            elseif (isset($_SERVER['REQUEST_URI'])) {
                $this->_requestUri = $_SERVER['REQUEST_URI'];
                if (!empty($_SERVER['HTTP_HOST'])) {
                    if (strpos($this->_requestUri, $_SERVER['HTTP_HOST']) !== false)
                        $this->_requestUri = preg_replace('/^\w+:\/\/[^\/]+/', '', $this->_requestUri);
                }
                else
                    $this->_requestUri = preg_replace('/^(http|https):\/\/[^\/]+/i', '', $this->_requestUri);
            }
            elseif (isset($_SERVER['ORIG_PATH_INFO'])) {                  $this->_requestUri = $_SERVER['ORIG_PATH_INFO'];
                if (!empty($_SERVER['QUERY_STRING']))
                    $this->_requestUri .= '?' . $_SERVER['QUERY_STRING'];
            }
            else
                throw new \Exception(App::t('app', 'HttpRequest is unable to determine the request URI.'));
        }

        return $this->_requestUri;
    }

    
    public function getRequestType() {
        if (isset($_POST['_method']))
            return strtoupper($_POST['_method']);
        elseif (isset($_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE']))
            return strtoupper($_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE']);

        return strtoupper(isset($_SERVER['REQUEST_METHOD']) ? $_SERVER['REQUEST_METHOD'] : 'GET');
    }

    
    public function getPluginUrl() {
        return rtrim($this->_pluginUrl, '/');
    }

    
    public function setPluginUrl($pluginUrl) {
        $this->_pluginUrl = $pluginUrl;
    }

    
    public function end($status = 0, $exit = true) {
        if ($exit) {
            exit($status);
        }
    }

    
    public static function log($msg, $level = 'info', $category = 'application') {
        if (ENVOQON_DEBUG && ENVOQON_TRACE_LEVEL > 0 && $level !== Logger::LEVEL_PROFILE) {
            $traces = debug_backtrace();
            $count = 0;
            foreach ($traces as $trace) {
                if (isset($trace['file'], $trace['line']) && strpos($trace['file'], App_PATH) !== 0) {
                    $msg .= "\nin " . $trace['file'] . ' (' . $trace['line'] . ')';
                    if (++$count >= ENVOQON_TRACE_LEVEL)
                        break;
                }
            }
        }
        error_log($msg);
    }

    
    public static function trace($msg, $category = 'application') {
        if (ENVOQON_DEBUG) {
            error_log($msg);
        }
    }

    
    public static function t($category, $message, $params = array(), $source = null, $language = null) {
        if ($params === array()) {
            return $message;
        }
        if (!is_array($params)) {
            $params = array($params);
        }
        if (isset($params[0])) {             if (strpos($message, '|') !== false) {
                if (strpos($message, '#') === false) {
                    $chunks = explode('|', $message);
                    $expressions = array(
                        0 => 'n==1',
                        1 => 'true',
                    );
                    if ($n = min(count($chunks), count($expressions))) {
                        for ($i = 0; $i < $n; $i++) {
                            $chunks[$i] = $expressions[$i] . '#' . $chunks[$i];
                        }
                        $message = implode('|', $chunks);
                    }
                }
                $message = ChoiceFormat::format($message, $params[0]);
            }
            if (!isset($params['{n}'])) {
                $params['{n}'] = $params[0];
            }
            unset($params[0]);
        }
        return $params !== array() ? strtr($message, $params) : $message;
    }

}
