<?php
namespace envoqon\base;

use envoqon\App;

class Component {

    private $_e;

    
    public function __get($name) {
        $getter = 'get' . $name;
        if (method_exists($this, $getter)) {
            return $this->$getter();
        }
        elseif (strncasecmp($name, 'on', 2) === 0 && method_exists($this, $name)) {
                        $name = strtolower($name);
            if (!isset($this->_e[$name])) {
                $this->_e[$name] = new CList;
            }
            return $this->_e[$name];
        }
        throw new \Exception(App::t('app', 'Property "{class}.{property}" is not defined.', array('{class}' => get_class($this), '{property}' => $name)));
    }

    
    public function __set($name, $value) {
        $setter = 'set' . $name;
        if (method_exists($this, $setter)) {
            return $this->$setter($value);
        }
        elseif (strncasecmp($name, 'on', 2) === 0 && method_exists($this, $name)) {
                        $name = strtolower($name);
            if (!isset($this->_e[$name])) {
                $this->_e[$name] = new CList;
            }
            return $this->_e[$name]->add($value);
        }
        if (method_exists($this, 'get' . $name)) {
            throw new \Exception(App::t('app', 'Property "{class}.{property}" is read only.', array('{class}' => get_class($this), '{property}' => $name)));
        }
        else {
            throw new \Exception(App::t('app', 'Property "{class}.{property}" is not defined.', array('{class}' => get_class($this), '{property}' => $name)));
        }
    }

    
    public function __isset($name) {
        $getter = 'get' . $name;
        if (method_exists($this, $getter)) {
            return $this->$getter() !== null;
        }
        elseif (strncasecmp($name, 'on', 2) === 0 && method_exists($this, $name)) {
            $name = strtolower($name);
            return isset($this->_e[$name]) && $this->_e[$name]->getCount();
        }
        return false;
    }

    
    public function __unset($name) {
        $setter = 'set' . $name;
        if (method_exists($this, $setter)) {
            $this->$setter(null);
        }
        elseif (strncasecmp($name, 'on', 2) === 0 && method_exists($this, $name)) {
            unset($this->_e[strtolower($name)]);
        }
        elseif (method_exists($this, 'get' . $name)) {
            throw new \Exception(App::t('app', 'Property "{class}.{property}" is read only.', array('{class}' => get_class($this), '{property}' => $name)));
        }
    }

    
    public function __call($name, $parameters) {
        if (class_exists('Closure', false) && ($this->canGetProperty($name) || property_exists($this, $name)) && $this->$name instanceof Closure) {
            return call_user_func_array($this->$name, $parameters);
        }
        throw new \Exception(App::t('app', '{class} and its behaviors do not have a method or closure named "{name}".', array('{class}' => get_class($this), '{name}' => $name)));
    }

    
    public function hasProperty($name) {
        return method_exists($this, 'get' . $name) || method_exists($this, 'set' . $name);
    }

    
    public function canGetProperty($name) {
        return method_exists($this, 'get' . $name);
    }

    
    public function canSetProperty($name) {
        return method_exists($this, 'set' . $name);
    }

    
    public function hasEvent($name) {
        return !strncasecmp($name, 'on', 2) && method_exists($this, $name);
    }

    
    public function hasEventHandler($name) {
        $name = strtolower($name);
        return isset($this->_e[$name]) && $this->_e[$name]->getCount() > 0;
    }

    
    public function getEventHandlers($name) {
        if ($this->hasEvent($name)) {
            $name = strtolower($name);
            if (!isset($this->_e[$name])) {
                $this->_e[$name] = new CList;
            }
            return $this->_e[$name];
        }
        else
            throw new \Exception(App::t('app', 'Event "{class}.{event}" is not defined.', array('{class}' => get_class($this), '{event}' => $name)));
    }

    
    public function attachEventHandler($name, $handler) {
        $this->getEventHandlers($name)->add($handler);
    }

    
    public function detachEventHandler($name, $handler) {
        if ($this->hasEventHandler($name)) {
            return $this->getEventHandlers($name)->remove($handler) !== false;
        }
        else {
            return false;
        }
    }

    
    public function raiseEvent($name, $event) {
        $name = strtolower($name);
        if (isset($this->_e[$name])) {
            foreach ($this->_e[$name] as $handler) {
                if (is_string($handler)) {
                    call_user_func($handler, $event);
                }
                elseif (is_callable($handler, true)) {
                    if (is_array($handler)) {
                                                list($object, $method) = $handler;
                        if (is_string($object)) {                             call_user_func($handler, $event);
                        }
                        elseif (method_exists($object, $method)) {
                            $object->$method($event);
                        }
                        else {
                            throw new \Exception(App::t('app', 'Event "{class}.{event}" is attached with an invalid handler "{handler}".', array('{class}' => get_class($this), '{event}' => $name, '{handler}' => $handler[1])));
                        }
                    }
                    else {                         call_user_func($handler, $event);
                    }
                }
                else {
                    throw new \Exception(App::t('app', 'Event "{class}.{event}" is attached with an invalid handler "{handler}".', array('{class}' => get_class($this), '{event}' => $name, '{handler}' => gettype($handler))));
                }
                                if (($event instanceof CEvent) && $event->handled) {
                    return;
                }
            }
        }
        elseif (ENVOQON_DEBUG && !$this->hasEvent($name)) {
            throw new \Exception(App::t('app', 'Event "{class}.{event}" is not defined.', array('{class}' => get_class($this), '{event}' => $name)));
        }
    }

    
    public function evaluateExpression($_expression_, $_data_ = array()) {
        if (is_string($_expression_)) {
            extract($_data_);
            return eval('return ' . $_expression_ . ';');
        }
        else {
            $_data_[] = $this;
            return call_user_func_array($_expression_, $_data_);
        }
    }

}


class Event extends Component {

    
    public $sender;

    
    public $handled = false;

    
    public $params;

    
    public function __construct($sender = null, $params = null) {
        $this->sender = $sender;
        $this->params = $params;
    }

}


class Enumerable {
    
}
