<?php



namespace envoqon\db;

use envoqon\db\DbConnection;
use envoqon\bb\DbCriteria;
use envoqon\App;
use envoqon\db\DbException;

abstract class ActiveRecord extends \envoqon\base\Model {

    
    public static $db;
    private static $_models = array();       private static $_md = array();        private $_new = false;          protected $_attributes = array();        private $_c;            private $_pk;            private $_alias = 't';      
    

    public function __construct($scenario = 'insert') {
        if ($scenario === null)             return;

        $this->setScenario($scenario);
        $this->setIsNewRecord(true);
        $this->_attributes = $this->getMetaData()->attributeDefaults;

        $this->init();
        $this->afterConstruct();
    }

    
    public function init() {
        
    }

    
    public function __sleep() {
        return array_keys((array) $this);
    }

    
    public function __get($name) {
        if (isset($this->_attributes[$name])) {
            return $this->_attributes[$name];
        }
        elseif (isset($this->getMetaData()->columns[$name])) {
            return null;
        }
        else {
            return parent::__get($name);
        }
    }

    
    public function __set($name, $value) {
        if ($this->setAttribute($name, $value) === false) {
            parent::__set($name, $value);
        }
    }

    
    public function __isset($name) {
        if (isset($this->_attributes[$name]))
            return true;
        elseif (isset($this->getMetaData()->columns[$name]))
            return false;
        else
            return parent::__isset($name);
    }

    
    public function __unset($name) {
        if (isset($this->getMetaData()->columns[$name]))
            unset($this->_attributes[$name]);
        else
            parent::__unset($name);
    }

    
    public function __call($name, $parameters) {
        $scopes = $this->scopes();
        if (isset($scopes[$name])) {
            $this->getDbCriteria()->mergeWith($scopes[$name]);
            return $this;
        }

        return parent::__call($name, $parameters);
    }

    
    public function getDbCriteria($createIfNull = true) {
        if ($this->_c === null) {
            if (($c = $this->defaultScope()) !== array() || $createIfNull)
                $this->_c = new DbCriteria($c);
        }
        return $this->_c;
    }

    
    public function setDbCriteria($criteria) {
        $this->_c = $criteria;
    }

    
    public function defaultScope() {
        return array();
    }

    
    public function resetScope($resetDefault = true) {
        if ($resetDefault)
            $this->_c = new DbCriteria();
        else
            $this->_c = null;

        return $this;
    }

    
    public static function model($className = __CLASS__) {
        if (isset(self::$_models[$className]))
            return self::$_models[$className];
        else {
            $model = self::$_models[$className] = new $className(null);
            return $model;
        }
    }

    
    public function getMetaData() {
        $className = get_class($this);
        if (!array_key_exists($className, self::$_md)) {
            self::$_md[$className] = null;             self::$_md[$className] = new ActiveRecordMetaData($this);
        }
        return self::$_md[$className];
    }

    
    public function refreshMetaData() {
        $className = get_class($this);
        if (array_key_exists($className, self::$_md))
            unset(self::$_md[$className]);
    }

    
    public function tableName() {
        $tableName = get_class($this);
        if (($pos = strrpos($tableName, '\\')) !== false)
            return substr($tableName, $pos + 1);
        return $tableName;
    }

    
    public function primaryKey() {
        
    }

    
    public function scopes() {
        return array();
    }

    
    public function attributeNames() {
        return array_keys($this->getMetaData()->columns);
    }

    
    public function getAttributeLabel($attribute) {
        $labels = $this->attributeLabels();
        if (isset($labels[$attribute]))
            return $labels[$attribute];
        else
            return $this->generateAttributeLabel($attribute);
    }

    
    public function getDbConnection() {
        if (self::$db !== null)
            return self::$db;
        else {
            self::$db = App::getDb();
            if (self::$db instanceof DbConnection)
                return self::$db;
            else
                throw new DbException(App::t('app', 'Active Record requires a "db" DbConnection application component.'));
        }
    }

    
    public function getTableSchema() {
        return $this->getMetaData()->tableSchema;
    }

    
    public function getCommandBuilder() {
        return $this->getDbConnection()->getSchema()->getCommandBuilder();
    }

    
    public function hasAttribute($name) {
        return isset($this->getMetaData()->columns[$name]);
    }

    
    public function getAttribute($name) {
        if (property_exists($this, $name))
            return $this->$name;
        elseif (isset($this->_attributes[$name]))
            return $this->_attributes[$name];
    }

    
    public function setAttribute($name, $value) {
        if (property_exists($this, $name))
            $this->$name = $value;
        elseif (isset($this->getMetaData()->columns[$name]))
            $this->_attributes[$name] = $value;
        else
            return false;
        return true;
    }

    
    public function getAttributes($names = true) {
        $attributes = $this->_attributes;
        foreach ($this->getMetaData()->columns as $name => $column) {
            if (property_exists($this, $name)) {
                $attributes[$name] = $this->$name;
            }
            elseif ($names === true && !isset($attributes[$name])) {
                $attributes[$name] = null;
            }
        }
        if (is_array($names)) {
            $attrs = array();
            foreach ($names as $name) {
                if (property_exists($this, $name))
                    $attrs[$name] = $this->$name;
                else
                    $attrs[$name] = isset($attributes[$name]) ? $attributes[$name] : null;
            }
            return $attrs;
        }
        else
            return $attributes;
    }

    
    public function save($runValidation = true, $attributes = null) {
        if (!$runValidation || $this->validate($attributes))
            return $this->getIsNewRecord() ? $this->insert($attributes) : $this->update($attributes);
        else
            return false;
    }

    
    public function getIsNewRecord() {
        return $this->_new;
    }

    
    public function setIsNewRecord($value) {
        $this->_new = $value;
    }

    
    public function onBeforeSave($event) {
        $this->raiseEvent('onBeforeSave', $event);
    }

    
    public function onAfterSave($event) {
        $this->raiseEvent('onAfterSave', $event);
    }

    
    public function onBeforeDelete($event) {
        $this->raiseEvent('onBeforeDelete', $event);
    }

    
    public function onAfterDelete($event) {
        $this->raiseEvent('onAfterDelete', $event);
    }

    
    public function onBeforeFind($event) {
        $this->raiseEvent('onBeforeFind', $event);
    }

    
    public function onAfterFind($event) {
        $this->raiseEvent('onAfterFind', $event);
    }

    
    public function onBeforeCount($event) {
        $this->raiseEvent('onBeforeCount', $event);
    }

    
    protected function beforeSave() {
        if ($this->hasEventHandler('onBeforeSave')) {
            $event = new CModelEvent($this);
            $this->onBeforeSave($event);
            return $event->isValid;
        }
        else
            return true;
    }

    
    protected function afterSave() {
        if ($this->hasEventHandler('onAfterSave'))
            $this->onAfterSave(new CEvent($this));
    }

    
    protected function beforeDelete() {
        if ($this->hasEventHandler('onBeforeDelete')) {
            $event = new CModelEvent($this);
            $this->onBeforeDelete($event);
            return $event->isValid;
        }
        else
            return true;
    }

    
    protected function afterDelete() {
        if ($this->hasEventHandler('onAfterDelete'))
            $this->onAfterDelete(new CEvent($this));
    }

    
    protected function beforeFind() {
        if ($this->hasEventHandler('onBeforeFind')) {
            $event = new CModelEvent($this);
            $this->onBeforeFind($event);
        }
    }

    
    protected function beforeCount() {
        if ($this->hasEventHandler('onBeforeCount'))
            $this->onBeforeCount(new CEvent($this));
    }

    
    protected function afterFind() {
        if ($this->hasEventHandler('onAfterFind'))
            $this->onAfterFind(new CEvent($this));
    }

    
    public function beforeFindInternal() {
        $this->beforeFind();
    }

    
    public function afterFindInternal() {
        $this->afterFind();
    }

    
    public function insert($attributes = null) {
        if (!$this->getIsNewRecord())
            throw new DbException(App::t('app', 'The active record cannot be inserted to database because it is not new.'));
        if ($this->beforeSave()) {
            App::trace(get_class($this) . '.insert()', 'system.db.ar.CActiveRecord');
            $builder = $this->getCommandBuilder();
            $table = $this->getTableSchema();
            $command = $builder->createInsertCommand($table, $this->getAttributes($attributes));
            if ($command->execute()) {
                $primaryKey = $table->primaryKey;
                if ($table->sequenceName !== null) {
                    if (is_string($primaryKey) && $this->$primaryKey === null)
                        $this->$primaryKey = $builder->getLastInsertID($table);
                    elseif (is_array($primaryKey)) {
                        foreach ($primaryKey as $pk) {
                            if ($this->$pk === null) {
                                $this->$pk = $builder->getLastInsertID($table);
                                break;
                            }
                        }
                    }
                }
                $this->_pk = $this->getPrimaryKey();
                $this->afterSave();
                $this->setIsNewRecord(false);
                $this->setScenario('update');
                return true;
            }
        }
        return false;
    }

    
    public function update($attributes = null) {
        if ($this->getIsNewRecord())
            throw new DbException(App::t('app', 'The active record cannot be updated because it is new.'));
        if ($this->beforeSave()) {
            App::trace(get_class($this) . '.update()', 'system.db.ar.CActiveRecord');
            if ($this->_pk === null)
                $this->_pk = $this->getPrimaryKey();
            $this->updateByPk($this->getOldPrimaryKey(), $this->getAttributes($attributes));
            $this->_pk = $this->getPrimaryKey();
            $this->afterSave();
            return true;
        }
        else
            return false;
    }

    
    public function saveAttributes($attributes) {
        if (!$this->getIsNewRecord()) {
            App::trace(get_class($this) . '.saveAttributes()', 'system.db.ar.CActiveRecord');
            $values = array();
            foreach ($attributes as $name => $value) {
                if (is_integer($name))
                    $values[$value] = $this->$value;
                else
                    $values[$name] = $this->$name = $value;
            }
            if ($this->_pk === null)
                $this->_pk = $this->getPrimaryKey();
            if ($this->updateByPk($this->getOldPrimaryKey(), $values) > 0) {
                $this->_pk = $this->getPrimaryKey();
                return true;
            }
            else
                return false;
        }
        else
            throw new DbException(App::t('app', 'The active record cannot be updated because it is new.'));
    }

    
    public function saveCounters($counters) {
        App::trace(get_class($this) . '.saveCounters()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $table = $this->getTableSchema();
        $criteria = $builder->createPkCriteria($table, $this->getOldPrimaryKey());
        $command = $builder->createUpdateCounterCommand($this->getTableSchema(), $counters, $criteria);
        if ($command->execute()) {
            foreach ($counters as $name => $value)
                $this->$name = $this->$name + $value;
            return true;
        }
        else
            return false;
    }

    
    public function delete() {
        if (!$this->getIsNewRecord()) {
            App::trace(get_class($this) . '.delete()', 'system.db.ar.CActiveRecord');
            if ($this->beforeDelete()) {
                $result = $this->deleteByPk($this->getPrimaryKey()) > 0;
                $this->afterDelete();
                return $result;
            }
            else
                return false;
        }
        else
            throw new DbException(App::t('app', 'The active record cannot be deleted because it is new.'));
    }

    
    public function refresh() {
        App::trace(get_class($this) . '.refresh()', 'system.db.ar.CActiveRecord');
        if (($record = $this->findByPk($this->getPrimaryKey())) !== null) {
            $this->_attributes = array();
            foreach ($this->getMetaData()->columns as $name => $column) {
                if (property_exists($this, $name))
                    $this->$name = $record->$name;
                else
                    $this->_attributes[$name] = $record->$name;
            }
            return true;
        }
        else
            return false;
    }

    
    public function equals($record) {
        return $this->tableName() === $record->tableName() && $this->getPrimaryKey() === $record->getPrimaryKey();
    }

    
    public function getPrimaryKey() {
        $table = $this->getTableSchema();
        if (is_string($table->primaryKey))
            return $this->{$table->primaryKey};
        elseif (is_array($table->primaryKey)) {
            $values = array();
            foreach ($table->primaryKey as $name)
                $values[$name] = $this->$name;
            return $values;
        }
        else
            return null;
    }

    
    public function setPrimaryKey($value) {
        $this->_pk = $this->getPrimaryKey();
        $table = $this->getTableSchema();
        if (is_string($table->primaryKey))
            $this->{$table->primaryKey} = $value;
        elseif (is_array($table->primaryKey)) {
            foreach ($table->primaryKey as $name)
                $this->$name = $value[$name];
        }
    }

    
    public function getOldPrimaryKey() {
        return $this->_pk;
    }

    
    public function setOldPrimaryKey($value) {
        $this->_pk = $value;
    }

    
    protected function query($criteria, $all = false) {
        $this->beforeFind();
        $this->applyScopes($criteria);
        if (!$all)
            $criteria->limit = 1;
        $command = $this->getCommandBuilder()->createFindCommand($this->getTableSchema(), $criteria);
        return $all ? $this->populateRecords($command->queryAll(), true, $criteria->index) : $this->populateRecord($command->queryRow());
    }

    
    public function applyScopes(&$criteria) {
        if (!empty($criteria->scopes)) {
            $scs = $this->scopes();
            $c = $this->getDbCriteria();
            foreach ((array) $criteria->scopes as $k => $v) {
                if (is_integer($k)) {
                    if (is_string($v)) {
                        if (isset($scs[$v])) {
                            $c->mergeWith($scs[$v], true);
                            continue;
                        }
                        $scope = $v;
                        $params = array();
                    }
                    elseif (is_array($v)) {
                        $scope = key($v);
                        $params = current($v);
                    }
                }
                elseif (is_string($k)) {
                    $scope = $k;
                    $params = $v;
                }

                call_user_func_array(array($this, $scope), (array) $params);
            }
        }

        if (isset($c) || ($c = $this->getDbCriteria(false)) !== null) {
            $c->mergeWith($criteria);
            $criteria = $c;
            $this->resetScope(false);
        }
    }

    
    public function getTableAlias($quote = false, $checkScopes = true) {
        if ($checkScopes && ($criteria = $this->getDbCriteria(false)) !== null && $criteria->alias != '')
            $alias = $criteria->alias;
        else
            $alias = $this->_alias;
        return $quote ? $this->getDbConnection()->getSchema()->quoteTableName($alias) : $alias;
    }

    
    public function setTableAlias($alias) {
        $this->_alias = $alias;
    }

    
    public function find($condition = '', $params = array()) {
        App::trace(get_class($this) . '.find()', 'system.db.ar.CActiveRecord');
        $criteria = $this->getCommandBuilder()->createCriteria($condition, $params);
        return $this->query($criteria);
    }

    
    public function findAll($condition = '', $params = array()) {
        App::trace(get_class($this) . '.findAll()', 'system.db.ar.CActiveRecord');
        $criteria = $this->getCommandBuilder()->createCriteria($condition, $params);
        return $this->query($criteria, true);
    }

    
    public function findByPk($pk, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.findByPk()', 'system.db.ar.CActiveRecord');
        $prefix = $this->getTableAlias(true) . '.';
        $criteria = $this->getCommandBuilder()->createPkCriteria($this->getTableSchema(), $pk, $condition, $params, $prefix);
        return $this->query($criteria);
    }

    
    public function findAllByPk($pk, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.findAllByPk()', 'system.db.ar.CActiveRecord');
        $prefix = $this->getTableAlias(true) . '.';
        $criteria = $this->getCommandBuilder()->createPkCriteria($this->getTableSchema(), $pk, $condition, $params, $prefix);
        return $this->query($criteria, true);
    }

    
    public function findByAttributes($attributes, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.findByAttributes()', 'system.db.ar.CActiveRecord');
        $prefix = $this->getTableAlias(true) . '.';
        $criteria = $this->getCommandBuilder()->createColumnCriteria($this->getTableSchema(), $attributes, $condition, $params, $prefix);
        return $this->query($criteria);
    }

    
    public function findAllByAttributes($attributes, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.findAllByAttributes()', 'system.db.ar.CActiveRecord');
        $prefix = $this->getTableAlias(true) . '.';
        $criteria = $this->getCommandBuilder()->createColumnCriteria($this->getTableSchema(), $attributes, $condition, $params, $prefix);
        return $this->query($criteria, true);
    }

    
    public function findBySql($sql, $params = array()) {
        App::trace(get_class($this) . '.findBySql()', 'system.db.ar.CActiveRecord');
        $this->beforeFind();
        $command = $this->getCommandBuilder()->createSqlCommand($sql, $params);
        return $this->populateRecord($command->queryRow());
    }

    
    public function findAllBySql($sql, $params = array()) {
        App::trace(get_class($this) . '.findAllBySql()', 'system.db.ar.CActiveRecord');
        $this->beforeFind();
        $command = $this->getCommandBuilder()->createSqlCommand($sql, $params);
        return $this->populateRecords($command->queryAll());
    }

    
    public function count($condition = '', $params = array()) {
        App::trace(get_class($this) . '.count()', 'system.db.ar.CActiveRecord');
        $this->beforeCount();
        $builder = $this->getCommandBuilder();
        $criteria = $builder->createCriteria($condition, $params);
        $this->applyScopes($criteria);
        return $builder->createCountCommand($this->getTableSchema(), $criteria)->queryScalar();
    }

    
    public function countByAttributes($attributes, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.countByAttributes()', 'system.db.ar.CActiveRecord');
        $prefix = $this->getTableAlias(true) . '.';
        $builder = $this->getCommandBuilder();
        $this->beforeCount();
        $criteria = $builder->createColumnCriteria($this->getTableSchema(), $attributes, $condition, $params, $prefix);
        $this->applyScopes($criteria);
        return $builder->createCountCommand($this->getTableSchema(), $criteria)->queryScalar();
    }

    
    public function countBySql($sql, $params = array()) {
        App::trace(get_class($this) . '.countBySql()', 'system.db.ar.CActiveRecord');
        $this->beforeCount();
        return $this->getCommandBuilder()->createSqlCommand($sql, $params)->queryScalar();
    }

    
    public function exists($condition = '', $params = array()) {
        App::trace(get_class($this) . '.exists()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $criteria = $builder->createCriteria($condition, $params);
        $table = $this->getTableSchema();
        $criteria->select = '1';
        $criteria->limit = 1;
        $this->applyScopes($criteria);
        return $builder->createFindCommand($table, $criteria, $this->getTableAlias(false, false))->queryRow() !== false;
    }

    
    public function updateByPk($pk, $attributes, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.updateByPk()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $table = $this->getTableSchema();
        $criteria = $builder->createPkCriteria($table, $pk, $condition, $params);
        $command = $builder->createUpdateCommand($table, $attributes, $criteria);
        return $command->execute();
    }

    
    public function updateAll($attributes, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.updateAll()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $criteria = $builder->createCriteria($condition, $params);
        $command = $builder->createUpdateCommand($this->getTableSchema(), $attributes, $criteria);
        return $command->execute();
    }

    
    public function updateCounters($counters, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.updateCounters()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $criteria = $builder->createCriteria($condition, $params);
        $command = $builder->createUpdateCounterCommand($this->getTableSchema(), $counters, $criteria);
        return $command->execute();
    }

    
    public function deleteByPk($pk, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.deleteByPk()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $criteria = $builder->createPkCriteria($this->getTableSchema(), $pk, $condition, $params);
        $command = $builder->createDeleteCommand($this->getTableSchema(), $criteria);
        return $command->execute();
    }

    
    public function deleteAll($condition = '', $params = array()) {
        App::trace(get_class($this) . '.deleteAll()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $criteria = $builder->createCriteria($condition, $params);
        $command = $builder->createDeleteCommand($this->getTableSchema(), $criteria);
        return $command->execute();
    }

    
    public function deleteAllByAttributes($attributes, $condition = '', $params = array()) {
        App::trace(get_class($this) . '.deleteAllByAttributes()', 'system.db.ar.CActiveRecord');
        $builder = $this->getCommandBuilder();
        $table = $this->getTableSchema();
        $criteria = $builder->createColumnCriteria($table, $attributes, $condition, $params);
        $command = $builder->createDeleteCommand($table, $criteria);
        return $command->execute();
    }

    
    public function populateRecord($attributes, $callAfterFind = true) {
        if ($attributes !== false) {
            $record = $this->instantiate($attributes);
            $record->setScenario('update');
            $record->init();
            $md = $record->getMetaData();
            foreach ($attributes as $name => $value) {
                if (property_exists($record, $name))
                    $record->$name = $value;
                elseif (isset($md->columns[$name]))
                    $record->_attributes[$name] = $value;
            }
            $record->_pk = $record->getPrimaryKey();
            if ($callAfterFind)
                $record->afterFind();
            return $record;
        }
        else
            return null;
    }

    
    public function populateRecords($data, $callAfterFind = true, $index = null) {
        $records = array();
        foreach ($data as $attributes) {
            if (($record = $this->populateRecord($attributes, $callAfterFind)) !== null) {
                if ($index === null)
                    $records[] = $record;
                else
                    $records[$record->$index] = $record;
            }
        }
        return $records;
    }

    
    protected function instantiate($attributes) {
        $class = get_class($this);
        $model = new $class(null);
        return $model;
    }

    
    public function offsetExists($offset) {
        return $this->__isset($offset);
    }
   
    public function toArray() {
        $fields = $this->resolveFields();
        $attributes = $this->_attributes;
        foreach ($this->getMetaData()->columns as $name => $column) {
            if (property_exists($this, $name)) {
                $attributes[$name] = $this->$name;
            }
            elseif (!isset($attributes[$name])) {
                $attributes[$name] = null;
            }
        }
        if (is_array($fields)) {
            $attrs = array();
            foreach ($fields as $key => $name) {
                if (property_exists($this, $key)) {
                    $attrs[$name] = $this->$key;
                }
                elseif (isset($attributes[$key])){
                    $attrs[$name] =  $attributes[$key] ;
                }
                else {
                    $getter = 'get' . $key;
                    if (method_exists($this, $getter)) {
                        $attrs[$name] = $this->$getter();
                    }
                }
                
            }
            return $attrs;
        }
        else {
            return [];
        }
    }
    
}


class ActiveRecordMetaData
{
	
	public $tableSchema;
	
	public $columns;

	
	public $attributeDefaults=array();

	private $_modelClassName;

	
	public function __construct($model)
	{
		$this->_modelClassName=get_class($model);

		$tableName=$model->tableName();
		if(($table=$model->getDbConnection()->getSchema()->getTable($tableName))===null)
			throw new DbException(App::t('app','The table "{table}" for active record class "{class}" cannot be found in the database.',
				array('{class}'=>$this->_modelClassName,'{table}'=>$tableName)));
				
		if(($modelPk=$model->primaryKey())!==null || $table->primaryKey===null)
		{
			$table->primaryKey=$modelPk;
			if(is_string($table->primaryKey) && isset($table->columns[$table->primaryKey]))
				$table->columns[$table->primaryKey]->isPrimaryKey=true;
			elseif(is_array($table->primaryKey))
			{
				foreach($table->primaryKey as $name)
				{
					if(isset($table->columns[$name]))
						$table->columns[$name]->isPrimaryKey=true;
				}
			}
		}
		$this->tableSchema=$table;
		$this->columns=$table->columns;

		foreach($table->columns as $name=>$column)
		{
			if(!$column->isPrimaryKey && $column->defaultValue!==null)
				$this->attributeDefaults[$name]=$column->defaultValue;
		}
	}

}
