<?php

namespace envoqon\db;


abstract class CustomPost extends \envoqon\db\ActiveRecord{   
    
    public function getPrimaryKey() {
        return 'ID';
    }

    
    public function tableName() {
        return '{{posts}}';
    }
    
    public function createPostType() {
        $this->createPostTypeImp($this->getSlug(), $this->getSingularLabel(), $this->getPluralLabel(), 'page', $this->getSupports());
        add_action('save_post', array($this, 'saveMetaInfo'), 10, 3);
        $this->afterCustomPostCrated();
    }
    
    protected function afterCustomPostCrated() {
        
    }

    public function saveMetaInfo($post_id, $post, $update) {
        $post_type = get_post_type($post_id);
        if ($this->getSlug() != $post_type) {
            return;
        }
        $this->ID = $post_id;
        if($this->validate()){
            $this->savePost($update);
        }        
    }

    public function save($runValidation = true, $attributes = null) {
        throw new Exception("Save not implemented for custom post");
    }
    
    public function update($attributes = null) {
        throw new \Exception("Update not implemented for custom post");
    }
    
    public function setMeta($name,$value) {
        update_post_meta($this->ID, $name, $value);
    }
    
    public function getMeta($name) {
        return get_post_meta($this->ID, $name, true);
    }
    
    
    protected abstract function savePost($update);

    protected abstract function getSingularLabel();

    protected abstract function getPluralLabel();

    protected abstract function getSlug();

    protected function getSupports() {
        return array(
            'title',
            'editor',
            'thumbnail',
            'excerpt',
                        );
    }

    
    private function createPostTypeImp($slug, $singularLabel, $pluralLabel, $capability, $supports) {
        $labels = array(
            'name' => __($pluralLabel),
            'singular_name' => __($singularLabel),
            'add_new' => __('Add ' . $singularLabel),
            'add_new_item' => __('Add New ' . $singularLabel),
            'edit_item' => __('Edit ' . $singularLabel),
            'new_item' => __('New ' . $singularLabel),
            'view_item' => __('View ' . $singularLabel),
            'search_items' => __('Search ' . $pluralLabel),
            'not_found' => __('No ' . $pluralLabel . ' Found'),
            'not_found_in_trash' => __('No ' . $pluralLabel . ' found in Trash')
        );
        $args = array(
            'labels' => $labels,
            'capability_type' => $capability,
            'public' => true,
            'rewrite' => false,
            'show_in_menu' => false,
            'supports' => $supports
        );
        register_post_type($slug, $args);
    }

}
