<?php



namespace envoqon\db;

use envoqon\App;
use envoqon\db\DbConnection;
use Exception;

abstract class DbMigration extends \envoqon\base\Component
{
	private $_db;

	
	public function up()
	{
		$transaction=$this->getDbConnection()->beginTransaction();
		try
		{
			if($this->safeUp()===false)
			{
				$transaction->rollback();
				return false;
			}
			$transaction->commit();
		}
		catch(Exception $e)
		{
			$transaction->rollback();
			return false;
		}
	}

	
	public function down()
	{
		$transaction=$this->getDbConnection()->beginTransaction();
		try
		{
			if($this->safeDown()===false)
			{
				$transaction->rollback();
				return false;
			}
			$transaction->commit();
		}
		catch(Exception $e)
		{
			$transaction->rollback();
			return false;
		}
	}

	
	public function safeUp()
	{
	}

	
	public function safeDown()
	{
	}

	
	public function getDbConnection()
	{
		if($this->_db===null)
		{
			$this->_db=App::getDb();
			if(!$this->_db instanceof DbConnection)
				throw new Exception(App::t('app', 'The "db" application component must be configured to be a DbConnection object.'));
		}
		return $this->_db;
	}

	
	public function setDbConnection($db)
	{
		$this->_db=$db;
	}

	
	public function execute($sql, $params=array())
	{
		$this->getDbConnection()->createCommand($sql)->execute($params);
	}

	
	public function insert($table, $columns)
	{
		$this->getDbConnection()->createCommand()->insert($table, $columns);
	}

	
	public function insertMultiple($table, $data)
	{
		$builder=$this->getDbConnection()->getSchema()->getCommandBuilder();
		$command=$builder->createMultipleInsertCommand($table,$data);
		$command->execute();
	}

	
	public function update($table, $columns, $conditions='', $params=array())
	{
		$this->getDbConnection()->createCommand()->update($table, $columns, $conditions, $params);
	}

	
	public function delete($table, $conditions='', $params=array())
	{
		$this->getDbConnection()->createCommand()->delete($table, $conditions, $params);
	}

	
	public function createTable($table, $columns, $options=null)
	{
		$this->getDbConnection()->createCommand()->createTable($table, $columns, $options);
	}

	
	public function renameTable($table, $newName)
	{
		$this->getDbConnection()->createCommand()->renameTable($table, $newName);
	}

	
	public function dropTable($table)
	{
		$this->getDbConnection()->createCommand()->dropTable($table);
	}

	
	public function truncateTable($table)
	{
		$this->getDbConnection()->createCommand()->truncateTable($table);
	}

	
	public function addColumn($table, $column, $type)
	{
		$this->getDbConnection()->createCommand()->addColumn($table, $column, $type);
	}

	
	public function dropColumn($table, $column)
	{
		$this->getDbConnection()->createCommand()->dropColumn($table, $column);
	}

	
	public function renameColumn($table, $name, $newName)
	{
		$this->getDbConnection()->createCommand()->renameColumn($table, $name, $newName);
	}

	
	public function alterColumn($table, $column, $type)
	{
		$this->getDbConnection()->createCommand()->alterColumn($table, $column, $type);
	}

	
	public function addForeignKey($name, $table, $columns, $refTable, $refColumns, $delete=null, $update=null)
	{
		$this->getDbConnection()->createCommand()->addForeignKey($name, $table, $columns, $refTable, $refColumns, $delete, $update);
	}

	
	public function dropForeignKey($name, $table)
	{
		$this->getDbConnection()->createCommand()->dropForeignKey($name, $table);
	}

	
	public function createIndex($name, $table, $columns, $unique=false)
	{
		$this->getDbConnection()->createCommand()->createIndex($name, $table, $columns, $unique);
	}

	
	public function dropIndex($name, $table, $unique=false)
	{
		$this->getDbConnection()->createCommand()->dropIndex($name, $table,$unique);
	}

	
	public function refreshTableSchema($table)
	{
		$this->getDbConnection()->getSchema()->getTable($table,true);
	}

	
	public function addPrimaryKey($name,$table,$columns)
	{
		$this->getDbConnection()->createCommand()->addPrimaryKey($name,$table,$columns);
	}

	
	public function dropPrimaryKey($name,$table)
	{
		$this->getDbConnection()->createCommand()->dropPrimaryKey($name,$table);
	}
}

