<?php

namespace envoqon\db;




class MysqlColumnSchema extends \envoqon\base\Component{

    
    public $name;

    
    public $rawName;

    
    public $allowNull;

    
    public $dbType;

    
    public $type;

    
    public $defaultValue;

    
    public $size;

    
    public $precision;

    
    public $scale;

    
    public $isPrimaryKey;

    
    public $isForeignKey;

    
    public $autoIncrement = false;

    
    public $comment = '';

    
    public function init($dbType, $defaultValue) {
        $this->dbType = $dbType;
        $this->extractType($dbType);
        $this->extractLimit($dbType);
        if ($defaultValue !== null)
            $this->extractDefault($defaultValue);
    }

    
    protected function extractType($dbType) {
        if (strncmp($dbType, 'enum', 4) === 0)
            $this->type = 'string';
        elseif (strpos($dbType, 'float') !== false || strpos($dbType, 'double') !== false)
            $this->type = 'double';
        elseif (strpos($dbType, 'bool') !== false)
            $this->type = 'boolean';
        elseif (strpos($dbType, 'int') === 0 && strpos($dbType, 'unsigned') === false || preg_match('/(bit|tinyint|smallint|mediumint)/', $dbType))
            $this->type = 'integer';
        else
            $this->type = 'string';
    }

    
    protected function extractLimit($dbType) {
        if (strncmp($dbType, 'enum', 4) === 0 && preg_match('/\(([\'"])(.*)\\1\)/', $dbType, $matches)) {
                        $values = explode($matches[1] . ',' . $matches[1], $matches[2]);
            $size = 0;
            foreach ($values as $value) {
                if (($n = strlen($value)) > $size)
                    $size = $n;
            }
            $this->size = $this->precision = $size;
        }
        else {
            if(strpos($dbType,'(') && preg_match('/\((.*)\)/',$dbType,$matches))
		{
			$values=explode(',',$matches[1]);
			$this->size=$this->precision=(int)$values[0];
			if(isset($values[1]))
				$this->scale=(int)$values[1];
		}
        }
    }

    
    protected function extractDefault($defaultValue) {
        if (strncmp($this->dbType, 'bit', 3) === 0)
            $this->defaultValue = bindec(trim($defaultValue, 'b\''));
        elseif (($this->dbType === 'timestamp' || $this->dbType === 'datetime') && $defaultValue === 'CURRENT_TIMESTAMP')
            $this->defaultValue = null;
        else {
            $this->defaultValue=$this->typecast($defaultValue);
        }
    }

    
    public function typecast($value) {
        if (gettype($value) === $this->type || $value === null || $value instanceof \envoqon\db\DbExpression)
            return $value;
        if ($value === '' && $this->allowNull)
            return $this->type === 'string' ? '' : null;
        switch ($this->type) {
            case 'string': return (string) $value;
            case 'integer': return (integer) $value;
            case 'boolean': return (boolean) $value;
            case 'double':
            default: return $value;
        }
    }

}
