<?php
namespace envoqon\utils;



class Date {

    private $time;


    
    const PATTERN_DATETIME_SHORT = 'Y-m-d';

    
    const PATTERN_ENGLISH = 'l, d F, Y';

    
    const PATTERN_ENGLISH_SHORT = 'd M, Y';

    
    const PATTERN_ENGLISH_DATETIME = 'd M, Y H:i:s';

    
    const PATTERN_ISO_8601_DATE = 'Y-m-d';
    
    const PATTERN_ISO_8601_DATETIME = 'Y-m-d H:i:s';
    
    
    const PATTERN_ISO_8601_TIME = 'H:i:s';
    
    const PATTERN_YEAR = 'Y';
    
    public function __construct() {
        $num_args = func_num_args();
        if ($num_args > 0) {
            $args = func_get_args();
        }
        if ($num_args > 1) {
            $seconds = $minutes = $hours = $day = $month = $year = 0;
        }
        switch ($num_args) {
            case 6:
                $seconds = $args[5];
            case 5:
                $minutes = $args[4];
            case 4:
                $hours = $args[3];
            case 3:
                $day = $args[2];
            case 2:
                $month = $args[1];
                $year = $args[0];
                $this->time = mktime($hours, $minutes, $seconds, ($month + 1), $day, $year);
                break;
            case 1:
                if (is_int($args[0])) {
                    $this->time = $args[0];
                }
                elseif (is_string($args[0])) {
                    $this->time = strtotime($args[0]);
                }
                break;
            case 0:
                $this->time = mktime();
                break;
        }
    }

    public function instance() {
        return new Date(func_get_args());
    }

    public function formatTime($format) {
        return self::format($this->time, $format);
    }

    
    public static function format($time, $format, $addTime = false) {
        if ($time) {
            try {
                return $addTime ? @date($format . ' ' . self::PATTERN_TIME, $time) : @date($format, $time);
            }
            catch (Exception $e) {
                return null;
            }
        }
        return null;
    }

    
    public static function formatDateTime($date, $format) {
        if ($date) {
            try {
                return @date_format(date_create(str_replace(',', '', $date)), $format);             }
            catch (Exception $e) {
                return null;
            }
        }
        return null;
    }

    public static function getCurrentTime($pattern) {
        return date($pattern, time());
    }

    public static function convertDaysToTime($days) {
        return $days * 24 * 60 * 60;
    }

    public static function convertTimeToDays($timestamp) {
        return (int) ceil($timestamp / 24 * 60 * 60);
    }

    public static function timezones() {
        $regions = DateTimeZone::listIdentifiers();
        $list = array();
        foreach ($regions as $i => $name) {
            $list[$name] = $name;
        }
        return $list;
    }

    public static function datePatterns() {
        $time = time();
        $format = array(
            'd F, Y' => date('d F, Y', $time),             'd M, Y' => date('d M, Y', $time),             'd/m/Y' => date('d-m-Y', $time),             'F d, Y' => date('F d, Y', $time),             'M d, Y' => date('M d, Y', $time),             'm/d/Y' => date('m/d/Y', $time),         );
        return $format;
    }

    public static function convertToJavascript($format) {
        return strtr($format, array(
            'F' => 'MM', 'm' => 'mm', 'd' => 'dd', 'Y' => 'yy'
        ));
    }
    
    public static function convertToMoment($format) {
        return strtr($format, array(
            'F' => 'MMMM','M'=>'MMM', 'm' => 'M', 'd' => 'DD', 'Y' => 'YYYY'
        ));
    }
    
    public static function getMonthName($monthnumber) {
        return date("F",mktime(0,0,0,(int)$monthnumber,1,2011));
    }

    
    public static function parse($date) {
        $t = strtotime($date);
        return $t === false ? '' : $t;
    }

    public function getDate() {
        return (int) date("j", $this->time);
    }

    public function getDay() {
        return (int) date("w", $this->time);
    }

    public function getFullYear() {
        return (int) date("Y", $this->time);
    }

    public function getHours() {
        return (int) date("H", $this->time);
    }

    public function getMinutes() {
        return (int) date("i", $this->time);
    }

    public function getMonth() {
        $temp = (int) date("n", $this->time);
        return --$temp;
    }

    public function getSeconds() {
        return (int) date("s", $this->time);
    }

    public function getTime() {
        return $this->time;
    }
    
    
    public static function getAge($datetime){
        $now = new DatTime('now');
        $date = new DateTime($datetime);
        $interval = $now->diff($date);
        return $interval->format("%y years %m months and %d days");
    }

}
