<?php



namespace envoqon\validators;

use envoqon\App;
use envoqon\web\UploadedFile;


class FileValidator extends \envoqon\validators\Validator
{
	
	public $allowEmpty=false;
	
	public $types;
	
	public $mimeTypes;
	
	public $minSize;
	
	public $maxSize;
	
	public $tooLarge;
	
	public $tooSmall;
	
	public $wrongType;
	
	public $wrongMimeType;
	
	public $maxFiles=1;
	
	public $tooMany;

	
	protected function validateAttribute($object, $attribute)
	{
		$files=$object->$attribute;
		if($this->maxFiles > 1)
		{
			if(!is_array($files) || !isset($files[0]) || !$files[0] instanceof UploadedFile)
				$files = UploadedFile::getInstances($object, $attribute);
			if(array()===$files)
				return $this->emptyAttribute($object, $attribute);
			if(count($files) > $this->maxFiles)
			{
				$message=$this->tooMany!==null?$this->tooMany : App::t('app', '{attribute} cannot accept more than {limit} files.');
				$this->addError($object, $attribute, $message, array('{attribute}'=>$attribute, '{limit}'=>$this->maxFiles));
			}
			else
				foreach($files as $file)
					$this->validateFile($object, $attribute, $file);
		}
		else
		{
			if (is_array($files))
			{
				if (count($files) > 1)
				{
					$message=$this->tooMany!==null?$this->tooMany : App::t('app', '{attribute} cannot accept more than {limit} files.');
					$this->addError($object, $attribute, $message, array('{attribute}'=>$attribute, '{limit}'=>$this->maxFiles));
					return;
				}
				else
					$file = empty($files) ? null : reset($files);
			}
			else
				$file = $files;
			if(!$file instanceof UploadedFile)
			{
				$file = UploadedFile::getInstance($object, $attribute);
				if(null===$file)
					return $this->emptyAttribute($object, $attribute);
			}
			$this->validateFile($object, $attribute, $file);
		}
	}

	
	protected function validateFile($object, $attribute, $file)
	{
		$error=(null===$file ? null : $file->getError());
		if($error==UPLOAD_ERR_INI_SIZE || $error==UPLOAD_ERR_FORM_SIZE || $this->maxSize!==null && $file->getSize()>$this->maxSize)
		{
			$message=$this->tooLarge!==null?$this->tooLarge : App::t('app','The file "{file}" is too large. Its size cannot exceed {limit} bytes.');
			$this->addError($object,$attribute,$message,array('{file}'=>CHtml::encode($file->getName()), '{limit}'=>$this->getSizeLimit()));
			if($error!==UPLOAD_ERR_OK)
				return;
		}
		elseif($error!==UPLOAD_ERR_OK)
		{
			if($error==UPLOAD_ERR_NO_FILE)
				return $this->emptyAttribute($object, $attribute);
			elseif($error==UPLOAD_ERR_PARTIAL)
				throw new CException(App::t('app','The file "{file}" was only partially uploaded.',array('{file}'=>CHtml::encode($file->getName()))));
			elseif($error==UPLOAD_ERR_NO_TMP_DIR)
				throw new CException(App::t('app','Missing the temporary folder to store the uploaded file "{file}".',array('{file}'=>CHtml::encode($file->getName()))));
			elseif($error==UPLOAD_ERR_CANT_WRITE)
				throw new CException(App::t('app','Failed to write the uploaded file "{file}" to disk.',array('{file}'=>CHtml::encode($file->getName()))));
			elseif(defined('UPLOAD_ERR_EXTENSION') && $error==UPLOAD_ERR_EXTENSION)  				throw new CException(App::t('app','A PHP extension stopped the file upload.'));
			else
				throw new CException(App::t('app','Unable to upload the file "{file}" because of an unrecognized error.',array('{file}'=>CHtml::encode($file->getName()))));
		}

		if($this->minSize!==null && $file->getSize()<$this->minSize)
		{
			$message=$this->tooSmall!==null?$this->tooSmall : App::t('app','The file "{file}" is too small. Its size cannot be smaller than {limit} bytes.');
			$this->addError($object,$attribute,$message,array('{file}'=>CHtml::encode($file->getName()), '{limit}'=>$this->minSize));
		}

		if($this->types!==null)
		{
			if(is_string($this->types))
				$types=preg_split('/[\s,]+/',strtolower($this->types),-1,PREG_SPLIT_NO_EMPTY);
			else
				$types=$this->types;
			if(!in_array(strtolower($file->getExtensionName()),$types))
			{
				$message=$this->wrongType!==null?$this->wrongType : App::t('app','The file "{file}" cannot be uploaded. Only files with these extensions are allowed: {extensions}.');
				$this->addError($object,$attribute,$message,array('{file}'=>CHtml::encode($file->getName()), '{extensions}'=>implode(', ',$types)));
			}
		}

		if($this->mimeTypes!==null && !empty($file->tempName))
		{
			if(function_exists('finfo_open'))
			{
				$mimeType=false;
				if($info=finfo_open(defined('FILEINFO_MIME_TYPE') ? FILEINFO_MIME_TYPE : FILEINFO_MIME))
					$mimeType=finfo_file($info,$file->getTempName());
			}
			elseif(function_exists('mime_content_type'))
				$mimeType=mime_content_type($file->getTempName());
			else
				throw new CException(App::t('app','In order to use MIME-type validation provided by CFileValidator fileinfo PECL extension should be installed.'));

			if(is_string($this->mimeTypes))
				$mimeTypes=preg_split('/[\s,]+/',strtolower($this->mimeTypes),-1,PREG_SPLIT_NO_EMPTY);
			else
				$mimeTypes=$this->mimeTypes;

			if($mimeType===false || !in_array(strtolower($mimeType),$mimeTypes))
			{
				$message=$this->wrongMimeType!==null?$this->wrongMimeType : App::t('app','The file "{file}" cannot be uploaded. Only files of these MIME-types are allowed: {mimeTypes}.');
				$this->addError($object,$attribute,$message,array('{file}'=>CHtml::encode($file->getName()), '{mimeTypes}'=>implode(', ',$mimeTypes)));
			}
		}
	}

	
	protected function emptyAttribute($object, $attribute)
	{
		if($this->safe) 
			$object->$attribute=null;

		if(!$this->allowEmpty)
		{
			$message=$this->message!==null?$this->message : App::t('app','{attribute} cannot be blank.');
			$this->addError($object,$attribute,$message);
		}
	}

	
	protected function getSizeLimit()
	{
		$limit=ini_get('upload_max_filesize');
		$limit=$this->sizeToBytes($limit);
		if($this->maxSize!==null && $limit>0 && $this->maxSize<$limit)
			$limit=$this->maxSize;
		if(isset($_POST['MAX_FILE_SIZE']) && $_POST['MAX_FILE_SIZE']>0 && $_POST['MAX_FILE_SIZE']<$limit)
			$limit=$_POST['MAX_FILE_SIZE'];
		return $limit;
	}

	
	public function sizeToBytes($sizeStr)
	{
				switch (strtolower(substr($sizeStr, -1)))
		{
			case 'm': return (int)$sizeStr * 1048576; 			case 'k': return (int)$sizeStr * 1024; 			case 'g': return (int)$sizeStr * 1073741824; 			default: return (int)$sizeStr; 		}
	}
}
