<?php



namespace envoqon\validators;

use envoqon\App;
use envoqon\db\DbCriteria;
use envoqon\db\ActiveRecord;
use envoqon\web\Html;

class UniqueValidator extends \envoqon\validators\Validator
{
	
	public $caseSensitive=true;
	
	public $allowEmpty=true;
	
	public $className;
	
	public $attributeName;
	
	public $criteria=array();
	
	public $message;
	
	public $skipOnError=true;


	
	protected function validateAttribute($object,$attribute)
	{
		$value=$object->$attribute;
		if($this->allowEmpty && $this->isEmpty($value))
			return;

		if(is_array($value))
		{
						$this->addError($object,$attribute,App::t('app','{attribute} is invalid.'));
			return;
		}
                if($this->className===null){
                    $className=get_class($object);
                }
		$attributeName=$this->attributeName===null?$attribute:$this->attributeName;
		$finder=$this->getModel($className);
		$table=$finder->getTableSchema();
		if(($column=$table->getColumn($attributeName))===null)
			throw new \Exception(App::t('app','Table "{table}" does not have a column named "{column}".',
				array('{column}'=>$attributeName,'{table}'=>$table->name)));

		$columnName=$column->rawName;
		$criteria=new DbCriteria();
		if($this->criteria!==array())
			$criteria->mergeWith($this->criteria);
		$tableAlias = empty($criteria->alias) ? $finder->getTableAlias(true) : $criteria->alias;
		$valueParamName = DbCriteria::PARAM_PREFIX.DbCriteria::$paramCount++;
		$criteria->addCondition($this->caseSensitive ? "{$tableAlias}.{$columnName}={$valueParamName}" : "LOWER({$tableAlias}.{$columnName})=LOWER({$valueParamName})");
		$criteria->params[$valueParamName] = $value;

		if(!$object instanceof ActiveRecord || $object->isNewRecord || $object->tableName()!==$finder->tableName())
			$exists=$finder->exists($criteria);
		else
		{
			$criteria->limit=2;
			$objects=$finder->findAll($criteria);
			$n=count($objects);
			if($n===1)
			{
				if($column->isPrimaryKey)  					$exists=$object->getOldPrimaryKey()!=$object->getPrimaryKey();
				else
				{
										$exists=array_shift($objects)->getPrimaryKey()!=$object->getOldPrimaryKey();
				}
			}
			else
				$exists=$n>1;
		}

		if($exists)
		{
			$message=$this->message!==null?$this->message:App::t('app','{attribute} "{value}" has already been taken.');
			$this->addError($object,$attribute,$message,array('{value}'=>Html::encode($value)));
		}
	}
	
	
	protected function getModel($className)
	{
		return ActiveRecord::model($className);
	}
}

