<?php



namespace envoqon\web;

use envoqon\db\DbCriteria;
use envoqon\web\Html;
use envoqon\db\ActiveRecord;


class ActiveDataProvider extends \envoqon\web\DataProvider
{
	
	public $modelClass;
	
	public $model;
	
	public $keyAttribute;

	
	protected $_criteria;
	
	protected $_countCriteria;

	
	public function __construct($modelClass,$config=array())
	{
		if(is_string($modelClass))
		{
			$this->modelClass=$modelClass;
			$this->model=$this->getModel($this->modelClass);
		}
		elseif($modelClass instanceof ActiveRecord)
		{
			$this->modelClass=get_class($modelClass);
			$this->model=$modelClass;
		}
		$this->setId(Html::modelName($this->model));
		foreach($config as $key=>$value)
			$this->$key=$value;
	}

	
	public function getCriteria()
	{
		if($this->_criteria===null)
			$this->_criteria=new DbCriteria;
		return $this->_criteria;
	}

	
	public function setCriteria($value)
	{
		$this->_criteria=$value instanceof DbCriteria ? $value : new DbCriteria($value);
	}

	
	public function getCountCriteria()
	{
		if($this->_countCriteria===null)
			return $this->getCriteria();
		return $this->_countCriteria;
	}

	
	public function setCountCriteria($value)
	{
		$this->_countCriteria=$value instanceof DbCriteria ? $value : new DbCriteria($value);
	}

	
	public function getSort($className='envoqon\web\Sort')
	{
		if(($sort=parent::getSort($className))!==false)
			$sort->modelClass=$this->modelClass;
		return $sort;
	}

	
	protected function getModel($className)
	{
		return ActiveRecord::model($className);
	}

	
	protected function fetchData()
	{
		$criteria=clone $this->getCriteria();

		if(($pagination=$this->getPagination())!==false)
		{
			$pagination->setItemCount($this->getTotalItemCount());
			$pagination->applyLimit($criteria);
		}

		$baseCriteria=$this->model->getDbCriteria(false);

		if(($sort=$this->getSort())!==false)
		{
						if($baseCriteria!==null)
			{
				$c=clone $baseCriteria;
				$c->mergeWith($criteria);
				$this->model->setDbCriteria($c);
			}
			else
				$this->model->setDbCriteria($criteria);
			$sort->applyOrder($criteria);
		}

		$this->model->setDbCriteria($baseCriteria!==null ? clone $baseCriteria : null);
		$data=$this->model->findAll($criteria);
		$this->model->setDbCriteria($baseCriteria);  		return $data;
	}

	
	protected function fetchKeys()
	{
		$keys=array();
		foreach($this->getData() as $i=>$data)
		{
			$key=$this->keyAttribute===null ? $data->getPrimaryKey() : $data->{$this->keyAttribute};
			$keys[$i]=is_array($key) ? implode(',',$key) : $key;
		}
		return $keys;
	}

	
	protected function calculateTotalItemCount()
	{
		$baseCriteria=$this->model->getDbCriteria(false);
		if($baseCriteria!==null)
			$baseCriteria=clone $baseCriteria;
		$count=$this->model->count($this->getCountCriteria());
		$this->model->setDbCriteria($baseCriteria);
		return $count;
	}
}

