<?php



namespace envoqon\web;

use envoqon\db\ActiveRecord;
use envoqon\web\Html;
use envoqon\App;


class Sort extends \envoqon\base\Component {

    
    const SORT_ASC = false;

    
    const SORT_DESC = true;

    
    public $multiSort = false;

    
    public $modelClass;

    
    public $attributes = array();

    
    public $sortVar = 'sort';

    
    public $descTag = 'desc';

    
    public $defaultOrder;

    
    public $route = '';

    
    public $separators = array('-', '.');

    
    public $params;
    private $_directions;

    
    public function __construct($modelClass = null) {
        $this->modelClass = $modelClass;
    }

    
    public function applyOrder($criteria) {
        $order = $this->getOrderBy($criteria);
        if (!empty($order)) {
            if (!empty($criteria->order))
                $criteria->order .= ', ';
            $criteria->order .= $order;
        }
    }

    
    public function getOrderBy($criteria = null) {
        $directions = $this->getDirections();
        if (empty($directions))
            return is_string($this->defaultOrder) ? $this->defaultOrder : '';
        else {
            if ($this->modelClass !== null)
                $schema = $this->getModel($this->modelClass)->getDbConnection()->getSchema();
            $orders = array();
            foreach ($directions as $attribute => $descending) {
                $definition = $this->resolveAttribute($attribute);
                if (is_array($definition)) {
                    if ($descending)
                        $orders[] = isset($definition['desc']) ? (is_array($definition['desc']) ? implode(', ', $definition['desc']) : $definition['desc']) : $attribute . ' DESC';
                    else
                        $orders[] = isset($definition['asc']) ? (is_array($definition['asc']) ? implode(', ', $definition['asc']) : $definition['asc']) : $attribute;
                }
                elseif ($definition !== false) {
                    $attribute = $definition;
                    if (isset($schema)) {
                        if (($pos = strpos($attribute, '.')) !== false)
                            $attribute = $schema->quoteTableName(substr($attribute, 0, $pos)) . '.' . $schema->quoteColumnName(substr($attribute, $pos + 1));
                        else
                            $attribute = ($criteria === null || $criteria->alias === null ? $this->getModel($this->modelClass)->getTableAlias(true) : $schema->quoteTableName($criteria->alias)) . '.' . $schema->quoteColumnName($attribute);
                    }
                    $orders[] = $descending ? $attribute . ' DESC' : $attribute;
                }
            }
            return implode(', ', $orders);
        }
    }

    
    public function link($attribute, $label = null, $htmlOptions = array()) {
        if ($label === null)
            $label = $this->resolveLabel($attribute);
        if (($definition = $this->resolveAttribute($attribute)) === false)
            return $label;
        $directions = $this->getDirections();
        if (isset($directions[$attribute])) {
            $class = $directions[$attribute] ? 'desc' : 'asc';
            if (isset($htmlOptions['class']))
                $htmlOptions['class'] .= ' ' . $class;
            else
                $htmlOptions['class'] = $class;
            $descending = !$directions[$attribute];
            unset($directions[$attribute]);
        }
        elseif (is_array($definition) && isset($definition['default']))
            $descending = $definition['default'] === 'desc';
        else
            $descending = false;

        if ($this->multiSort)
            $directions = array_merge(array($attribute => $descending), $directions);
        else
            $directions = array($attribute => $descending);

        $url = $this->createUrl($directions);

        return $this->createLink($attribute, $label, $url, $htmlOptions);
    }

    
    public function resolveLabel($attribute) {
        $definition = $this->resolveAttribute($attribute);
        if (is_array($definition)) {
            if (isset($definition['label']))
                return $definition['label'];
        }
        elseif (is_string($definition))
            $attribute = $definition;
        if ($this->modelClass !== null)
            return $this->getModel($this->modelClass)->getAttributeLabel($attribute);
        else
            return $attribute;
    }

    
    public function getDirections() {
        if ($this->_directions === null) {
            $this->_directions = array();
            if (isset($_GET[$this->sortVar]) && is_string($_GET[$this->sortVar])) {
                $attributes = explode($this->separators[0], $_GET[$this->sortVar]);
                foreach ($attributes as $attribute) {
                    if (($pos = strrpos($attribute, $this->separators[1])) !== false) {
                        $descending = substr($attribute, $pos + 1) === $this->descTag;
                        if ($descending)
                            $attribute = substr($attribute, 0, $pos);
                    }
                    else
                        $descending = false;

                    if (($this->resolveAttribute($attribute)) !== false) {
                        $this->_directions[$attribute] = $descending;
                        if (!$this->multiSort)
                            return $this->_directions;
                    }
                }
            }
            if ($this->_directions === array() && is_array($this->defaultOrder))
                $this->_directions = $this->defaultOrder;
        }
        return $this->_directions;
    }

    
    public function getDirection($attribute) {
        $this->getDirections();
        return isset($this->_directions[$attribute]) ? $this->_directions[$attribute] : null;
    }

    
    public function createUrl($directions) {
        $sorts = array();
        foreach ($directions as $attribute => $descending) {
            $sorts[] = $descending ? $attribute . $this->separators[1] . $this->descTag : $attribute;
        }
        $params = $this->params === null ? $_GET : $this->params;
        $params[$this->sortVar] = implode($this->separators[0], $sorts);
        if(!$this->route){
            $c = App::instance()->getController();
            $this->route = $c->createAjaxUrl($c->getId(). '/'.$c->getActionId());
        }
        return add_query_arg($params,$this->route);
    }

    
    public function resolveAttribute($attribute) {
        if ($this->attributes !== array())
            $attributes = $this->attributes;
        elseif ($this->modelClass !== null)
            $attributes = $this->getModel($this->modelClass)->attributeNames();
        else
            return false;
        foreach ($attributes as $name => $definition) {
            if (is_string($name)) {
                if ($name === $attribute)
                    return $definition;
            }
            elseif ($definition === '*') {
                if ($this->modelClass !== null && $this->getModel($this->modelClass)->hasAttribute($attribute))
                    return $attribute;
            }
            elseif ($definition === $attribute)
                return $attribute;
        }
        return false;
    }

    
    protected function getModel($className) {
        return ActiveRecord::model($className);
    }

    
    protected function createLink($attribute, $label, $url, $htmlOptions) {
        return Html::link($label, $url, $htmlOptions);
    }

}
