<?php

namespace envoqon\web;

use envoqon\App;


class WebUser extends \envoqon\base\ApplicationComponent {

    const FLASH_COUNTERS = 'WebUser.flashcounters';
    const NOTICE_INFO = 'notice-info';
    const NOTICE_WARNING = 'notice-warning';
    const NOTICE_ERROR = 'notice-error';
    const NOTICE_SUCCESS = 'notice-success';

    public function __construct() {
        
    }

    public function init() {
        $this->updateFlash();
    }

    public function echoNotice() {
        $msg = '<div class="notice inline %1$s is-dismissible"><p>%2$s</p></div>';
        if ($this->hasFlash(self::NOTICE_INFO)) {
            printf($msg, self::NOTICE_INFO, $this->getFlash(self::NOTICE_INFO));
        }
        if ($this->hasFlash(self::NOTICE_WARNING)) {
            printf($msg, self::NOTICE_WARNING, $this->getFlash(self::NOTICE_WARNING));
        }
        if ($this->hasFlash(self::NOTICE_ERROR)) {
            printf($msg, self::NOTICE_ERROR, $this->getFlash(self::NOTICE_ERROR));
        }
        if ($this->hasFlash(self::NOTICE_SUCCESS)) {
            printf($msg, self::NOTICE_SUCCESS, $this->getFlash(self::NOTICE_SUCCESS));
        }
    }

    
    public static function getWpUser() {
        return wp_get_current_user();
    }

    
    public function getFlashes($delete = true) {
        $flashes = array();
        $prefix = App::instance()->getName();
        $keys = array_keys($_SESSION);
        $n = strlen($prefix);
        foreach ($keys as $key) {
            if (!strncmp($key, $prefix, $n)) {
                $flashes[substr($key, $n)] = $_SESSION[$key];
                if ($delete) {
                    unset($_SESSION[$key]);
                }
            }
        }
        if ($delete) {
            $this->setState(App::instance()->getName() . self::FLASH_COUNTERS, array());
        }
        return $flashes;
    }

    
    public function getFlash($key, $defaultValue = null, $delete = true) {
        $value = $this->getState(App::instance()->getName() . $key, $defaultValue);
        if ($delete) {
            $this->setFlash($key, null);
        }
        return $value;
    }

    
    public function setFlash($key, $value, $defaultValue = null) {
        $this->setState(App::instance()->getName() . $key, $value, $defaultValue);
        $counters = $this->getState(App::instance()->getName() . self::FLASH_COUNTERS, array());
        if ($value === $defaultValue) {
            unset($counters[$key]);
        }
        else {
            $counters[$key] = 0;
        }
        $this->setState(App::instance()->getName() . self::FLASH_COUNTERS, $counters, array());
    }

    
    public function hasFlash($key) {
        return $this->getFlash($key, null, false) !== null;
    }

    
    protected function updateFlash() {
        $counters = $this->getState(App::instance()->getName() . self::FLASH_COUNTERS);
        if (!is_array($counters)) {
            return;
        }
        foreach ($counters as $key => $count) {
            if ($count > 1) {
                unset($counters[$key]);
                $this->setState(App::instance()->getName() . $key, null);
            }
            else {
                $counters[$key] ++;
            }
        }
        $this->setState(App::instance()->getName() . self::FLASH_COUNTERS, $counters, array());
    }

    
    public function getState($key, $defaultValue = null) {
        return isset($_SESSION[$key]) ? $_SESSION[$key] : $defaultValue;
    }

    
    public function setState($key, $value, $defaultValue = null) {
        if ($value === $defaultValue) {
            unset($_SESSION[$key]);
        }
        else {
            $_SESSION[$key] = $value;
        }
    }

    public function checkAccess($role) {
        $user = wp_get_current_user();
        if (in_array($role, (array) $user->roles)) {
            return true;
        }
        return false;
    }
    
    public function getName(){
        return wp_get_current_user()->user_login;
    }

}
