<?php



namespace envoqon\web\filter;

use envoqon\App;
use envoqon\web\HttpException;


class AccessControlFilter extends Filter {

    
    public $message;
    private $_rules = array();

    
    public function getRules() {
        return $this->_rules;
    }

    
    public function setRules($rules) {
        foreach ($rules as $rule) {
            if (is_array($rule) && isset($rule[0])) {
                $r = new AccessRule;
                $r->allow = $rule[0] === 'allow';
                foreach (array_slice($rule, 1) as $name => $value) {
                    if ($name === 'expression' || $name === 'roles' || $name === 'message' || $name === 'deniedCallback')
                        $r->$name = $value;
                    else
                        $r->$name = array_map('strtolower', $value);
                }
                $this->_rules[] = $r;
            }
        }
    }

    
    protected function preFilter($filterChain) {
        $app = App::instance();
        $user = $app->getUser();
        $verb = $app->getRequestType();

        foreach ($this->getRules() as $rule) {
            if (($allow = $rule->isUserAllowed($user, $filterChain->controller, $verb)) > 0)                 break;
            elseif ($allow < 0) {                 if (isset($rule->deniedCallback))
                    call_user_func($rule->deniedCallback, $rule);
                else
                    $this->accessDenied($user, $this->resolveErrorMessage($rule));
                return false;
            }
        }

        return true;
    }

    
    protected function resolveErrorMessage($rule) {
        if ($rule->message !== null)
            return $rule->message;
        elseif ($this->message !== null)
            return $this->message;
        else
            return App::t('app', 'You are not authorized to perform this action.');
    }

    
    protected function accessDenied($user, $message) {
                App::instance()->getUser()->setFlash(\envoqon\web\WebUser::NOTICE_ERROR, "Permission denied.");
        $url = str_replace("/admin.php", "", App::instance()->adminBaseUrl);
        echo '<meta http-equiv="refresh" content="0;url=' . $url . '" />';
        App::instance()->end();
    }

}
