<?php

namespace envoqon\web\filter;

use envoqon\App;
use envoqon\collections\CList;
use envoqon\web\Controller;
use envoqon\web\InlineAction;


class FilterChain extends CList {

    
    public $controller;

    
    public $action;

    
    public $filterIndex = 0;

    
    public function __construct($controller, $action) {
        $this->controller = $controller;
        $this->action = $action;
    }

    
    public static function create($controller, $action, $filters) {
        $chain = new FilterChain($controller,$action);

        $actionID = $actionID = $action->getId();
        foreach ($filters as $filter) {
            if (is_string($filter)) {                  if (($pos = strpos($filter, '+')) !== false || ($pos = strpos($filter, '-')) !== false) {
                    $matched = preg_match("/\b{$actionID}\b/i", substr($filter, $pos + 1)) > 0;
                    if (($filter[$pos] === '+') === $matched)
                        $filter = InlineFilter::create($controller, trim(substr($filter, 0, $pos)));
                }
                else
                    $filter = InlineFilter::create($controller, $filter);
            }
            elseif (is_array($filter)) {                  if (!isset($filter[0]))
                    throw new \Exception(App::t('app', 'The first element in a filter configuration must be the filter class.'));
                $filterClass = $filter[0];
                unset($filter[0]);
                if (($pos = strpos($filterClass, '+')) !== false || ($pos = strpos($filterClass, '-')) !== false) {
                    $matched = preg_match("/\b{$actionID}\b/i", substr($filterClass, $pos + 1)) > 0;
                    if (($filterClass[$pos] === '+') === $matched)
                        $filterClass = trim(substr($filterClass, 0, $pos));
                    else
                        continue;
                }
                $filter['class'] = $filterClass;
                $filter = App::createComponent($filter);
            }

            if (is_object($filter)) {
                $filter->init();
                $chain->add($filter);
            }
        }
        return $chain;
    }

    
    public function insertAt($index, $item) {
        if ($item instanceof Filter)
            parent::insertAt($index, $item);
        else
            throw new \Exception(App::t('app', 'FilterChain can only take objects implementing the IFilter interface.'));
    }

    
    public function run() {
        if ($this->offsetExists($this->filterIndex)) {
            $filter = $this->itemAt($this->filterIndex++);
            App::trace('Running filter ' . ($filter instanceof InlineFilter ? get_class($this->controller) . '.filter' . $filter->name . '()' : get_class($filter) . '.filter()'), 'system.web.filters.FilterChain');
            $filter->filter($this);
        }
        else
            $this->controller->runAction($this->action);
    }

}
