<?php



namespace envoqon\widgets;

use envoqon\web\Html;
use envoqon\App;


abstract class BaseListView extends \envoqon\widgets\Widget
{
	
	public $dataProvider;
	
	public $tagName='div';
	
	public $htmlOptions=array();
	
	public $enableSorting=true;
	
	public $enablePagination=true;
	
	public $pager=array('class'=>'envoqon\widgets\LinkPager');
	
	public $template="{summary}\n{items}\n{pager}";
	
	public $summaryText;
	
	public $summaryTagName='div';
	
	public $emptyText;
	
	public $emptyTagName='span';
	
	public $emptyCssClass='empty';
	
	public $itemsCssClass='wp-list-table widefat fixed striped';
	
	public $summaryCssClass='summary';
	
	public $pagerCssClass='tablenav';
	
	public $loadingCssClass='loading';

	
	public function init()
	{
		if($this->dataProvider===null)
			throw new CException(App::t('zii','The "dataProvider" property cannot be empty.'));

		$this->dataProvider->getData();

		if(isset($this->htmlOptions['id']))
			$this->id=$this->htmlOptions['id'];
		else
			$this->htmlOptions['id']=$this->id;

		if($this->enableSorting && $this->dataProvider->getSort()===false)
			$this->enableSorting=false;
		if($this->enablePagination && $this->dataProvider->getPagination()===false)
			$this->enablePagination=false;
	}

	
	public function run()
	{
		$this->registerClientScript();

		echo Html::openTag($this->tagName,$this->htmlOptions)."\n";

		$this->renderContent();
		$this->renderKeys();

		echo Html::closeTag($this->tagName);
	}

	
	public function renderContent()
	{
		ob_start();
		echo preg_replace_callback("/{(\w+)}/",array($this,'renderSection'),$this->template);
		ob_end_flush();
	}

	
	protected function renderSection($matches)
	{
		$method='render'.$matches[1];
		if(method_exists($this,$method))
		{
			$this->$method();
			$html=ob_get_contents();
			ob_clean();
			return $html;
		}
		else
			return $matches[0];
	}

	
	public function renderEmptyText()
	{
		$emptyText=$this->emptyText===null ? App::t('zii','No results found.') : $this->emptyText;
		echo Html::tag($this->emptyTagName, array('class'=>$this->emptyCssClass), $emptyText);
	}

	
	public function renderKeys()
	{
		echo Html::openTag('div',array(
			'class'=>'keys',
			'style'=>'display:none',
			'title'=>App::instance()->baseUrl,
		));
		foreach($this->dataProvider->getKeys() as $key)
			echo "<span>".Html::encode($key)."</span>";
		echo "</div>\n";
	}

	
	public function renderSummary()
	{
		if(($count=$this->dataProvider->getItemCount())<=0)
			return;

		echo Html::openTag($this->summaryTagName, array('class'=>$this->summaryCssClass));
		if($this->enablePagination)
		{
			$pagination=$this->dataProvider->getPagination();
			$total=$this->dataProvider->getTotalItemCount();
			$start=$pagination->currentPage*$pagination->pageSize+1;
			$end=$start+$count-1;
			if($end>$total)
			{
				$end=$total;
				$start=$end-$count+1;
			}
			if(($summaryText=$this->summaryText)===null)
				$summaryText=App::t('zii','Displaying {start}-{end} of 1 result.|Displaying {start}-{end} of {count} results.',$total);
                        echo '<span class="displaying-num">';
			echo strtr($summaryText,array(
				'{start}'=>$start,
				'{end}'=>$end,
				'{count}'=>$total,
				'{page}'=>$pagination->currentPage+1,
				'{pages}'=>$pagination->pageCount,
			));
                        echo '</span>';
		}
		else
		{
			if(($summaryText=$this->summaryText)===null)
				$summaryText=App::t('zii','Total 1 result.|Total {count} results.',$count);
			echo strtr($summaryText,array(
				'{count}'=>$count,
				'{start}'=>1,
				'{end}'=>$count,
				'{page}'=>1,
				'{pages}'=>1,
			));
		}
		echo Html::closeTag($this->summaryTagName);
	}

	
	public function renderPager()
	{
		if(!$this->enablePagination)
			return;

		$pager=array();
		$class='';
		if(is_string($this->pager))
			$class=$this->pager;
		elseif(is_array($this->pager))
		{
			$pager=$this->pager;
			if(isset($pager['class']))
			{
				$class=$pager['class'];
				unset($pager['class']);
			}
		}
		$pager['pages']=$this->dataProvider->getPagination();

		if($pager['pages']->getPageCount()>1)
		{
			echo '<div class="'.$this->pagerCssClass.'">';
			App::instance()->getController()->widget($class,$pager);
			echo '</div>';
		}
		else
			App::instance()->getController()->widget($class,$pager);
	}

	
	public function registerClientScript()
	{
	}

	
	abstract public function renderItems();
}
