<?php



namespace envoqon\widgets;

use envoqon\web\Html;


abstract class GridColumn extends \envoqon\base\Component {

    
    public $id;

    
    public $grid;

    
    public $header;

    
    public $footer;

    
    public $visible = true;

    
    public $cssClassExpression;

    
    public $htmlOptions = array();

    
    public $filterHtmlOptions = array();

    
    public $headerHtmlOptions = array(
        'class' => 'manage-column'
    );

    
    public $footerHtmlOptions = array();

    
    public function __construct($grid) {
        $this->grid = $grid;
    }

    
    public function init() {
        
    }

    
    public function getHasFooter() {
        return $this->footer !== null;
    }

    
    public function renderFilterCell() {
        $this->renderFilterCellContent();
    }

    
    public function renderHeaderCell() {
        $this->headerHtmlOptions['id'] = $this->id;
        echo Html::openTag('th', $this->headerHtmlOptions);
        $this->renderHeaderCellContent();
        echo "</th>";
    }

    
    public function renderDataCell($row) {
        $data = $this->grid->dataProvider->data[$row];
        $options = $this->htmlOptions;
        if ($this->cssClassExpression !== null) {
            $class = $this->evaluateExpression($this->cssClassExpression, array('row' => $row, 'data' => $data));
            if (!empty($class)) {
                if (isset($options['class']))
                    $options['class'] .= ' ' . $class;
                else
                    $options['class'] = $class;
            }
        }
        echo Html::openTag('td', $options);
        $this->renderDataCellContent($row, $data);
        echo '</td>';
    }

    
    public function renderFooterCell() {
        echo Html::openTag('td', $this->footerHtmlOptions);
        $this->renderFooterCellContent();
        echo '</td>';
    }

    
    public function getHeaderCellContent() {
        return trim($this->header) !== '' ? $this->header : $this->grid->blankDisplay;
    }

    
    protected function renderHeaderCellContent() {
        echo $this->getHeaderCellContent();
    }

    
    public function getFooterCellContent() {
        if ($this->footer != null) {
            if (is_string($this->footer)) {
                return trim($this->footer) !== '' ? $this->footer : $this->grid->blankDisplay;
            }
            else {
                $fn = $this->footer;
                return $fn();
            }
        }
        return $this->getHeaderCellContent();
    }

    
    protected function renderFooterCellContent() {
        echo $this->getFooterCellContent();
    }

    
    public function getDataCellContent($row) {
        return $this->grid->blankDisplay;
    }

    
    protected function renderDataCellContent($row, $data) {
        echo $this->getDataCellContent($row);
    }

    
    public function getFilterCellContent() {
        return '';
    }

    
    protected function renderFilterCellContent() {
        echo $this->getFilterCellContent();
    }

}
