<?php



namespace envoqon\widgets;

use envoqon\widgets\PrimaryColumn;
use envoqon\widgets\DataColumn;
use envoqon\web\JavaScript;
use envoqon\web\JavaScriptExpression;
use envoqon\App;
use envoqon\web\ActiveDataProvider;
use envoqon\web\DataProvider;
use envoqon\web\Html;


class GridView extends \envoqon\widgets\BaseListView {

    private $_formatter;

    
    public $columns = array();

    
    public $rowCssClass = array('odd', 'even');

    
    public $rowCssClassExpression;

    
    public $rowHtmlOptionsExpression;

    
    public $showTableOnEmpty = true;

    
    public $ajaxUpdate;

    
    public $updateSelector = '{page}, {sort}';

    
    public $ajaxUpdateError;

    
    public $ajaxVar = 'ajax';

    
    public $ajaxUrl;

    
    public $ajaxType;

    
    public $beforeAjaxUpdate;

    
    public $afterAjaxUpdate;

    
    public $selectionChanged;

    
    public $selectableRows = 1;

    
    public $baseScriptUrl;

    
    public $cssFile;

    
    public $nullDisplay = '&nbsp;';

    
    public $blankDisplay = '&nbsp;';

    
    public $loadingCssClass = 'grid-view-loading';

    
    public $filterSelector = '{filter}';

    
    public $filterCssClass = 'tablenav';

    
    public $hideHeader = false;
    public $showSearchBox = true;

    
    public function init() {
        parent::init();

        if (empty($this->updateSelector))
            throw new \Exception(App::t('zii', 'The property updateSelector should be defined.'));
        if (empty($this->filterSelector))
            throw new \Exception(App::t('zii', 'The property filterSelector should be defined.'));

        if (!isset($this->htmlOptions['class']))
            $this->htmlOptions['class'] = 'grid-view';

        if ($this->baseScriptUrl === null) {
            $this->baseScriptUrl = App::instance()->pluginUrl . '/envoqon/widgets/assets/gridview';
        }

        if ($this->cssFile !== null) {
            $this->cssFile = $this->baseScriptUrl . '/styles.css';
            App::instance()->registerCssFile('gridview.css', $this->cssFile);
        }

        $this->initColumns();
    }

    public function run() {
        $this->renderSearch();
        parent::run();
    }

    
    protected function initColumns() {
        if ($this->columns === array()) {
            if ($this->dataProvider instanceof ActiveDataProvider)
                $this->columns = $this->dataProvider->model->attributeNames();
            elseif ($this->dataProvider instanceof DataProvider) {
                                $data = $this->dataProvider->getData();
                if (isset($data[0]) && is_array($data[0]))
                    $this->columns = array_keys($data[0]);
            }
        }
        $id = $this->getId();
        foreach ($this->columns as $i => $column) {
            if (is_string($column))
                $column = $this->createDataColumn($column);
            else {
                if (!isset($column['class']))
                    $column['class'] = 'envoqon\widgets\DataColumn';
                $column = App::createComponent($column, $this);
            }
            if (!$column->visible) {
                unset($this->columns[$i]);
                continue;
            }
            if ($column->id === null)
                $column->id = $id . '_c' . $i;
            $this->columns[$i] = $column;
        }

        foreach ($this->columns as $column)
            $column->init();
    }

    
    protected function createDataColumn($text) {
        if (!preg_match('/^([\w\.]+)(:(\w*))?(:(.*))?$/', $text, $matches))
            throw new \Exception(App::t('zii', 'The column must be specified in the format of "Name:Type:Label", where "Type" and "Label" are optional.'));
        $column = new DataColumn($this);
        $column->name = $matches[1];
        if (isset($matches[3]) && $matches[3] !== '')
            $column->type = $matches[3];
        if (isset($matches[5]))
            $column->header = $matches[5];
        return $column;
    }

    private function renderSearch() {
        if ($this->showSearchBox) {
            echo '<p class="search-box">'
            . '<label class="screen-reader-text" for="post-search-input">Search</label>'
            . '<input type="search" id="post-search-input" name="s" value="">'
            . '<input type="submit" id="search-submit" class="button" value="Search"></p>';
        }
    }

    
    public function registerClientScript() {
        $id = $this->getId();

        if ($this->ajaxUpdate === false)
            $ajaxUpdate = false;
        else
            $ajaxUpdate = array_unique(preg_split('/\s*,\s*/', $this->ajaxUpdate . ',' . $id, -1, PREG_SPLIT_NO_EMPTY));
        $options = array(
            'ajaxUpdate' => $ajaxUpdate,
            'ajaxVar' => $this->ajaxVar,
            'pagerClass' => $this->pagerCssClass,
            'loadingClass' => $this->loadingCssClass,
            'filterClass' => $this->filterCssClass,
            'tableClass' => $this->itemsCssClass,
            'selectableRows' => $this->selectableRows,
            'updateSelector' => $this->updateSelector,
            'filterSelector' => $this->filterSelector
        );
        if ($this->ajaxUrl !== null) {
            $options['url'] = Html::normalizeUrl($this->ajaxUrl);
        }
        else {
            $c = App::instance()->getController();
            $options['url'] = $c->createAjaxUrl($c->getId() . '/' . $c->getActionId());
        }
        if ($this->enablePagination) {
            $options['pageVar'] = $this->dataProvider->getPagination()->pageVar;
        }
        foreach (array('beforeAjaxUpdate', 'afterAjaxUpdate', 'ajaxUpdateError', 'selectionChanged') as $event) {
            if ($this->$event !== null) {
                if ($this->$event instanceof JavaScriptExpression)
                    $options[$event] = $this->$event;
                else
                    $options[$event] = new JavaScriptExpression($this->$event);
            }
        }

        $options = JavaScript::encode($options);
        App::instance()->registerScriptFile('gridview', $this->baseScriptUrl . '/jquery.yiigridview.js', array('jquery'));
        App::instance()->registerScriptFile('jquery.ba-bbq', $this->baseScriptUrl . '/jquery.ba-bbq.min.js', array('jquery'));
        App::instance()->registerScript(__CLASS__ . '#' . $id, "jQuery('#$id').yiiGridView($options);");
        App::instance()->registerScript(__CLASS__ . 'search', 'jQuery(".search-box #post-search-input").on("keypress",function(e) {if(e.which == 13) {jQuery(".search-box #search-submit").trigger("click");}});jQuery(".search-box #search-submit").on("click",function(e){
            $("#' . $this->id . '").yiiGridView("update",{"data":{q:$("#post-search-input").val()}});
        });');
        App::instance()->registerCss(__CLASS__ . 'css', '.grid-view-loading{background:url(' . $this->baseScriptUrl . '/loading.gif) no-repeat;}');
    }

    
    public function renderItems() {
        if ($this->dataProvider->getItemCount() > 0 || $this->showTableOnEmpty) {
            echo "<table class=\"{$this->itemsCssClass}\">\n";
            $this->renderTableHeader();
            ob_start();
            $this->renderTableBody();
            $body = ob_get_clean();
            $this->renderTableFooter();
            echo $body;             echo "</table>";
        }
        else
            $this->renderEmptyText();
    }

    
    public function renderTableHeader() {
        if (!$this->hideHeader) {
            echo "<thead>\n";

            echo "<tr>\n";
            foreach ($this->columns as $column)
                $column->renderHeaderCell();
            echo "</tr>\n";

            echo "</thead>\n";
        }
    }

    
    public function renderFilter() {
        ob_start();
        foreach ($this->columns as $column) {
            $column->renderFilterCell();
        }
        $content = ob_get_contents();
        ob_end_clean();
        if ($content) {
            echo '<div class="alignleft actions">';
            echo $content;
            echo '<input type="submit" name="filter_action" id="post-query-submit" class="button" value="Filter">';
            echo "</div>\n";
        }
    }

    
    public function renderTableFooter() {
        echo "<tfoot>\n";
        echo "<tr>\n";
        foreach ($this->columns as $column) {
            $column->renderFooterCell();
        }
        echo "</tr>\n";
        echo "</tfoot>\n";
    }

    
    public function renderTableBody() {
        $data = $this->dataProvider->getData();
        $n = count($data);
        echo "<tbody>\n";

        if ($n > 0) {
            for ($row = 0; $row < $n; ++$row)
                $this->renderTableRow($row);
        }
        else {
            echo '<tr><td colspan="' . count($this->columns) . '" class="empty">';
            $this->renderEmptyText();
            echo "</td></tr>\n";
        }
        echo "</tbody>\n";
    }

    
    public function renderTableRow($row) {
        $htmlOptions = array();
        if ($this->rowHtmlOptionsExpression !== null) {
            $data = $this->dataProvider->data[$row];
            $options = $this->evaluateExpression($this->rowHtmlOptionsExpression, array('row' => $row, 'data' => $data));
            if (is_array($options))
                $htmlOptions = $options;
        }

        if ($this->rowCssClassExpression !== null) {
            $data = $this->dataProvider->data[$row];
            $class = $this->evaluateExpression($this->rowCssClassExpression, array('row' => $row, 'data' => $data));
        }
        elseif (is_array($this->rowCssClass) && ($n = count($this->rowCssClass)) > 0)
            $class = $this->rowCssClass[$row % $n];

        if (!empty($class)) {
            if (isset($htmlOptions['class']))
                $htmlOptions['class'] .= ' ' . $class;
            else
                $htmlOptions['class'] = $class;
        }
        $htmlOptions['data-key'] = $this->dataProvider->keys[$row];

        echo Html::openTag('tr', $htmlOptions) . "\n";
        foreach ($this->columns as $column)
            $this->renderDataCell($column, $row);
        echo "</tr>\n";
    }

    
    protected function renderDataCell($column, $row) {
        $column->renderDataCell($row);
    }

    
    public function getHasFooter() {
        foreach ($this->columns as $column)
            if ($column->getHasFooter())
                return true;
        return false;
    }

    
    public function getFormatter() {
        if ($this->_formatter === null)
            $this->_formatter = App::instance()->format;
        return $this->_formatter;
    }

    
    public function setFormatter($value) {
        $this->_formatter = $value;
    }

    public function renderSummary() {
        echo "<div class=\"{$this->filterCssClass} top\">\n";
        $this->renderFilter();
        echo '<div class="tablenav-pages one-page">';
        parent::renderSummary();
        echo "</div>\n";
        echo "</div>\n";
    }

}
