<?php



namespace envoqon\widgets;

use envoqon\web\Pagination;
use envoqon\App;
use envoqon\web\Html;


class LinkPager extends \envoqon\widgets\Widget {

    const CSS_FIRST_PAGE = 'first';
    const CSS_LAST_PAGE = 'last';
    const CSS_PREVIOUS_PAGE = 'previous';
    const CSS_NEXT_PAGE = 'next';
    const CSS_INTERNAL_PAGE = 'page';
    const CSS_HIDDEN_PAGE = 'hidden';
    const CSS_SELECTED_PAGE = 'current-page';

    
    public $firstPageCssClass = self::CSS_FIRST_PAGE;

    
    public $lastPageCssClass = self::CSS_LAST_PAGE;

    
    public $previousPageCssClass = self::CSS_PREVIOUS_PAGE;

    
    public $nextPageCssClass = self::CSS_NEXT_PAGE;

    
    public $internalPageCssClass = self::CSS_INTERNAL_PAGE;

    
    public $hiddenPageCssClass = self::CSS_HIDDEN_PAGE;

    
    public $selectedPageCssClass = self::CSS_SELECTED_PAGE;

    
    public $maxButtonCount = 5;

    
    public $nextPageLabel;

    
    public $prevPageLabel;

    
    public $firstPageLabel;

    
    public $lastPageLabel;

    
    public $header;

    
    public $footer = '';

    
    public $cssFile;

    
    public $htmlOptions = array();

    
    private $_pages;

    
    public function init() {
        if ($this->nextPageLabel === null)
            $this->nextPageLabel = App::t('app', '&rsaquo;');
        if ($this->prevPageLabel === null)
            $this->prevPageLabel = App::t('app', '&lsaquo;');
        if ($this->firstPageLabel === null)
            $this->firstPageLabel = App::t('app', '&laquo;');
        if ($this->lastPageLabel === null)
            $this->lastPageLabel = App::t('app', '&raquo;');
        if ($this->header === null)
            $this->header = App::t('app', 'Go to page: ');

        if (!isset($this->htmlOptions['id']))
            $this->htmlOptions['id'] = $this->getId();
        if (!isset($this->htmlOptions['class']))
            $this->htmlOptions['class'] = 'tablenav-pages';
    }

    
    public function run() {
        $this->registerClientScript();
        $buttons = $this->createPageButtons();
        if (empty($buttons)) {
            return;
        }
        $header = '<span class="displaying-num">'. $this->header.'</span>';
        echo Html::tag('div', $this->htmlOptions, $header.implode("\n", $buttons));
        echo $this->footer;
    }

    
    protected function createPageButtons() {
        if (($pageCount = $this->getPageCount()) <= 1) {
            return array();
        }

        list($beginPage, $endPage) = $this->getPageRange();
        $currentPage = $this->getCurrentPage(false);         $buttons = array();
                if ($this->firstPageLabel !== false) {
            $buttons[] = $this->createPageButton($this->firstPageLabel, 0, $this->firstPageCssClass, $currentPage <= 0, false);
        }
                if ($this->prevPageLabel !== false) {
            if (($page = $currentPage - 1) < 0) {
                $page = 0;
            }
            $buttons[] = $this->createPageButton($this->prevPageLabel, $page, $this->previousPageCssClass, $currentPage <= 0, false);
        }

                for ($i = $beginPage; $i <= $endPage; ++$i)
            $buttons[] = $this->createPageButton($i + 1, $i, $this->internalPageCssClass, false, $i == $currentPage);

                if ($this->nextPageLabel !== false) {
            if (($page = $currentPage + 1) >= $pageCount - 1)
                $page = $pageCount - 1;
            $buttons[] = $this->createPageButton($this->nextPageLabel, $page, $this->nextPageCssClass, $currentPage >= $pageCount - 1, false);
        }
                if ($this->lastPageLabel !== false) {
            $buttons[] = $this->createPageButton($this->lastPageLabel, $pageCount - 1, $this->lastPageCssClass, $currentPage >= $pageCount - 1, false);
        }

        return $buttons;
    }

    
    protected function createPageButton($label, $page, $class, $hidden, $selected) {
        if ($hidden || $selected)
            $class .= ' ' . ($hidden ? $this->hiddenPageCssClass : $this->selectedPageCssClass);
        return Html::link($label, $this->createPageUrl($page),array('class'=>$class));
    }

    
    protected function getPageRange() {
        $currentPage = $this->getCurrentPage();
        $pageCount = $this->getPageCount();

        $beginPage = max(0, $currentPage - (int) ($this->maxButtonCount / 2));
        if (($endPage = $beginPage + $this->maxButtonCount - 1) >= $pageCount) {
            $endPage = $pageCount - 1;
            $beginPage = max(0, $endPage - $this->maxButtonCount + 1);
        }
        return array($beginPage, $endPage);
    }

    
    public function registerClientScript() {
        if ($this->cssFile !== false)
            self::registerCssFile($this->cssFile);
    }

    
    public static function registerCssFile($url = null) {
        if ($url == null) {
            App::instance()->registerCssFile('pager', $url);
        }
    }

    
    public function getPages() {
        if ($this->_pages === null)
            $this->_pages = $this->createPages();
        return $this->_pages;
    }

    
    public function setPages($pages) {
        $this->_pages = $pages;
    }

    
    protected function createPages() {
        return new Pagination;
    }

    
    public function getPageSize() {
        return $this->getPages()->getPageSize();
    }

    
    public function setPageSize($value) {
        $this->getPages()->setPageSize($value);
    }

    
    public function getItemCount() {
        return $this->getPages()->getItemCount();
    }

    
    public function setItemCount($value) {
        $this->getPages()->setItemCount($value);
    }

    
    public function getPageCount() {
        return $this->getPages()->getPageCount();
    }

    
    public function getCurrentPage($recalculate = true) {
        return $this->getPages()->getCurrentPage($recalculate);
    }

    
    public function setCurrentPage($value) {
        $this->getPages()->setCurrentPage($value);
    }

    
    protected function createPageUrl($page) {
        return $this->getPages()->createPageUrl($page);
    }

}
