<?php



namespace envoqon\widgets;

use envoqon\App;
use envoqon\web\Html;
use envoqon\web\JavaScriptExpression;
use envoqon\web\JavaScript;


class PrimaryColumn extends \envoqon\widgets\GridColumn {

    
    public $label = 'Link';

    
    public $labelExpression;

    
    public $url = '#';

    
    public $urlExpression;

    
    public $htmlOptions = array('class' => 'title column-title column-primary');

    
    public $footerHtmlOptions = array('class' => 'link-column');

    
    public $linkHtmlOptions = array('class' => 'row-title');

    
    public $headerHtmlOptions = array(
        'class' => 'manage-column column-primary'
    );
    public $disabledExpression;

    
    public $template = '{update}{delete}';

    
    public $buttons = array();
    public $click;

    
    public function init() {
        $this->initButtons();
        parent::init();
        $this->registerClientScript();
    }

    protected function initButtons() {
        foreach ($this->buttons as $id => $button) {
            if (strpos($this->template, '{' . $id . '}') === false) {
                unset($this->buttons[$id]);
            }
        }
        foreach ($this->buttons as $i => $button) {
            if (!isset($button['class'])) {
                $button['class'] = 'envoqon\widgets\ActionLink';
            }
            $button = App::createComponent($button, $this);
            if (!isset($button->options['class'])) {
                $button->options['class'] = $i;
            }
            $this->buttons[$i] = $button;
        }

        foreach ($this->buttons as $button) {
            $button->grid = $this->grid;
            $button->init();
        }
        if (!empty($this->buttons)) {
            $this->htmlOptions['class'] = $this->htmlOptions['class'] . ' has-row-actions';
        }
    }

    
    protected function registerClientScript() {
        $js = array();
        foreach ($this->buttons as $id => $button) {
            if ($button->click) {
                $click = $button->click instanceof JavaScriptExpression ? $button->click : new JavaScriptExpression($button->click);
                $function = JavaScript::encode($click);
                $class = preg_replace('/\s+/', '.', $button->options['class']);
                $js[] = "jQuery(document).on('click','#{$this->grid->id} a.{$class}',$function);";
            }
        }
        if ($this->click) {
            $function = JavaScript::encode(new JavaScriptExpression($this->click));
            $js[] = "jQuery(document).on('click','#{$this->grid->id} a.row-title',$function);";
        }
        if ($js !== array()) {
            App::instance()->registerScript(__CLASS__ . '#' . $this->id, implode("\n", $js));
        }
    }

    
    public function getDataCellContent($row) {
        $data = $this->grid->dataProvider->data[$row];
        $tr = array();
        $content = '<div class="row-actions">';
        foreach ($this->buttons as $id => $button) {
            $tr['{' . $id . '}'] = $button->render($id, $row, $data);
        }
        $content .= rtrim(strtr($this->template, $tr)," | ") . ' </div>';
        if ($this->urlExpression !== null)
            $url = $this->evaluateExpression($this->urlExpression, array('data' => $data, 'row' => $row));
        else
            $url = $this->url;
        if ($this->labelExpression !== null)
            $label = $this->evaluateExpression($this->labelExpression, array('data' => $data, 'row' => $row));
        else
            $label = $this->label;
        $options = $this->linkHtmlOptions;
        if ($this->disabledExpression) {
            $disabled = $this->evaluateExpression($this->disabledExpression, array('data' => $data, 'row' => $row));
            if ($disabled) {
                return '<strong>' . Html::tag('span', $options, $label) . '</strong>' . $content;
            }
        }
        return '<strong>' . Html::link($label, $url, $options) . '</strong>' . $content;
    }

}
